@@ -1,10 +1,14 @@
 use crate::{
     client::Client,
-    request::Request,
+    error::Error,
+    request::{Request, TryIntoRequest},
     response::{marker::EmptyBody, ResponseFuture},
     routing::Route,
 };
-use twilight_model::id::{ApplicationId, MessageId};
+use twilight_model::id::{
+    marker::{ApplicationMarker, MessageMarker},
+    Id,
+};
 
 /// Delete a followup message created from a interaction.
 ///
@@ -14,11 +18,14 @@ use twilight_model::id::{ApplicationId, MessageId};
 /// # #[tokio::main] async fn main() -> Result<(), Box<dyn std::error::Error>> {
 /// use std::env;
 /// use twilight_http::Client;
-/// use twilight_model::id::{MessageId, ApplicationId};
+/// use twilight_model::id::Id;
 ///
 /// let client = Client::new(env::var("DISCORD_TOKEN")?);
+/// let application_id = Id::new(1);
+///
 /// client
-///     .delete_followup_message("token here", MessageId::new(2).expect("non zero"))?
+///     .interaction(application_id)
+///     .delete_followup_message("token here", Id::new(2))
 ///     .exec()
 ///     .await?;
 /// # Ok(()) }
@@ -26,17 +33,17 @@ use twilight_model::id::{ApplicationId, MessageId};
 #[must_use = "requests must be configured and executed"]
 pub struct DeleteFollowupMessage<'a> {
     http: &'a Client,
-    message_id: MessageId,
+    message_id: Id<MessageMarker>,
     token: &'a str,
-    application_id: ApplicationId,
+    application_id: Id<ApplicationMarker>,
 }
 
 impl<'a> DeleteFollowupMessage<'a> {
     pub(crate) const fn new(
         http: &'a Client,
-        application_id: ApplicationId,
+        application_id: Id<ApplicationMarker>,
         token: &'a str,
-        message_id: MessageId,
+        message_id: Id<MessageMarker>,
     ) -> Self {
         Self {
             http,
@@ -46,42 +53,49 @@ impl<'a> DeleteFollowupMessage<'a> {
         }
     }
 
-    fn request(self) -> Request {
-        Request::builder(&Route::DeleteWebhookMessage {
+    /// Execute the request, returning a future resolving to a [`Response`].
+    ///
+    /// [`Response`]: crate::response::Response
+    pub fn exec(self) -> ResponseFuture<EmptyBody> {
+        let http = self.http;
+
+        match self.try_into_request() {
+            Ok(request) => http.request(request),
+            Err(source) => ResponseFuture::error(source),
+        }
+    }
+}
+
+impl TryIntoRequest for DeleteFollowupMessage<'_> {
+    fn try_into_request(self) -> Result<Request, Error> {
+        Ok(Request::builder(&Route::DeleteWebhookMessage {
             message_id: self.message_id.get(),
             thread_id: None,
             token: self.token,
             webhook_id: self.application_id.get(),
         })
         .use_authorization_token(false)
-        .build()
-    }
-
-    /// Execute the request, returning a future resolving to a [`Response`].
-    ///
-    /// [`Response`]: crate::response::Response
-    pub fn exec(self) -> ResponseFuture<EmptyBody> {
-        self.http.request(self.request())
+        .build())
     }
 }
 
 #[cfg(test)]
 mod tests {
     use super::DeleteFollowupMessage;
-    use crate::{client::Client, request::Request, routing::Route};
-    use twilight_model::id::{ApplicationId, MessageId};
+    use crate::{
+        client::Client,
+        request::{Request, TryIntoRequest},
+        routing::Route,
+    };
+    use std::error::Error;
+    use twilight_model::id::Id;
 
     #[test]
-    fn test_request() {
+    fn test_request() -> Result<(), Box<dyn Error>> {
         let client = Client::new("token".to_owned());
 
-        let builder = DeleteFollowupMessage::new(
-            &client,
-            ApplicationId::new(1).expect("non zero"),
-            "token",
-            MessageId::new(2).expect("non zero"),
-        );
-        let actual = builder.request();
+        let builder = DeleteFollowupMessage::new(&client, Id::new(1), "token", Id::new(2));
+        let actual = builder.try_into_request()?;
 
         let expected = Request::from_route(&Route::DeleteWebhookMessage {
             message_id: 2,
@@ -92,5 +106,7 @@ mod tests {
 
         assert_eq!(expected.path, actual.path);
         assert!(!actual.use_authorization_token());
+
+        Ok(())
     }
 }
