@@ -1,7 +1,16 @@
-use crate::{client::Client, request::Request, response::ResponseFuture, routing::Route};
+use crate::{
+    client::Client,
+    error::Error,
+    request::{Request, TryIntoRequest},
+    response::ResponseFuture,
+    routing::Route,
+};
 use twilight_model::{
     channel::Message,
-    id::{ApplicationId, MessageId},
+    id::{
+        marker::{ApplicationMarker, MessageMarker},
+        Id,
+    },
 };
 
 /// Get a followup message of an interaction.
@@ -14,31 +23,32 @@ use twilight_model::{
 /// use std::env;
 /// use twilight_http::Client;
 /// use twilight_http::request::AuditLogReason;
-/// use twilight_model::id::{ApplicationId, MessageId};
+/// use twilight_model::id::Id;
 ///
 /// let client = Client::new(env::var("DISCORD_TOKEN")?);
-/// client.set_application_id(ApplicationId::new(1).expect("non zero"));
+/// let application_id = Id::new(1);
 ///
 /// let response = client
-///     .followup_message("token here", MessageId::new(2).expect("non zero"))?
+///     .interaction(application_id)
+///     .followup_message("token here", Id::new(2))
 ///     .exec()
 ///     .await?;
 /// # Ok(()) }
 /// ```
 #[must_use = "requests must be configured and executed"]
 pub struct GetFollowupMessage<'a> {
-    application_id: ApplicationId,
+    application_id: Id<ApplicationMarker>,
     http: &'a Client,
-    message_id: MessageId,
+    message_id: Id<MessageMarker>,
     interaction_token: &'a str,
 }
 
 impl<'a> GetFollowupMessage<'a> {
     pub(crate) const fn new(
         http: &'a Client,
-        application_id: ApplicationId,
+        application_id: Id<ApplicationMarker>,
         interaction_token: &'a str,
-        message_id: MessageId,
+        message_id: Id<MessageMarker>,
     ) -> Self {
         Self {
             application_id,
@@ -48,56 +58,66 @@ impl<'a> GetFollowupMessage<'a> {
         }
     }
 
-    fn request(&self) -> Request {
-        Request::builder(&Route::GetFollowupMessage {
+    /// Execute the request, returning a future resolving to a [`Response`].
+    ///
+    /// [`Response`]: crate::response::Response
+    pub fn exec(self) -> ResponseFuture<Message> {
+        let http = self.http;
+
+        match self.try_into_request() {
+            Ok(request) => http.request(request),
+            Err(source) => ResponseFuture::error(source),
+        }
+    }
+}
+
+impl TryIntoRequest for GetFollowupMessage<'_> {
+    fn try_into_request(self) -> Result<Request, Error> {
+        Ok(Request::builder(&Route::GetFollowupMessage {
             application_id: self.application_id.get(),
             interaction_token: self.interaction_token,
             thread_id: None,
             message_id: self.message_id.get(),
         })
         .use_authorization_token(false)
-        .build()
-    }
-
-    /// Execute the request, returning a future resolving to a [`Response`].
-    ///
-    /// [`Response`]: crate::response::Response
-    pub fn exec(self) -> ResponseFuture<Message> {
-        self.http.request(self.request())
+        .build())
     }
 }
 
 #[cfg(test)]
 mod tests {
     use super::GetFollowupMessage;
-    use crate::{client::Client, request::Request, routing::Route};
+    use crate::{
+        client::Client,
+        request::{Request, TryIntoRequest},
+        routing::Route,
+    };
     use static_assertions::assert_impl_all;
     use std::error::Error;
-    use twilight_model::id::{ApplicationId, MessageId};
+    use twilight_model::id::{
+        marker::{ApplicationMarker, MessageMarker},
+        Id,
+    };
 
     assert_impl_all!(GetFollowupMessage<'_>: Send, Sync);
 
     #[test]
     fn test_request() -> Result<(), Box<dyn Error>> {
+        const APPLICATION_ID: Id<ApplicationMarker> = Id::new(1);
+        const MESSAGE_ID: Id<MessageMarker> = Id::new(2);
         const TOKEN: &str = "token";
 
-        fn application_id() -> ApplicationId {
-            ApplicationId::new(1).expect("non zero")
-        }
-
-        fn message_id() -> MessageId {
-            MessageId::new(2).expect("non zero")
-        }
-
         let client = Client::new("token".to_owned());
-        client.set_application_id(application_id());
 
-        let actual = client.followup_message(TOKEN, message_id())?.request();
+        let actual = client
+            .interaction(APPLICATION_ID)
+            .followup_message(TOKEN, MESSAGE_ID)
+            .try_into_request()?;
         let expected = Request::builder(&Route::GetFollowupMessage {
-            application_id: application_id().get(),
+            application_id: APPLICATION_ID.get(),
             interaction_token: TOKEN,
             thread_id: None,
-            message_id: message_id().get(),
+            message_id: MESSAGE_ID.get(),
         })
         .use_authorization_token(false)
         .build();
