@@ -1,16 +1,19 @@
 use crate::{
     client::Client,
     error::Error,
-    request::Request,
+    request::{Request, TryIntoRequest},
     response::{marker::EmptyBody, ResponseFuture},
     routing::Route,
 };
-use twilight_model::{application::callback::InteractionResponse, id::InteractionId};
+use twilight_model::{
+    application::callback::InteractionResponse,
+    id::{marker::InteractionMarker, Id},
+};
 
 /// Respond to an interaction, by ID and token.
 #[must_use = "requests must be configured and executed"]
 pub struct InteractionCallback<'a> {
-    interaction_id: InteractionId,
+    interaction_id: Id<InteractionMarker>,
     interaction_token: &'a str,
     response: &'a InteractionResponse,
     http: &'a Client,
@@ -19,7 +22,7 @@ pub struct InteractionCallback<'a> {
 impl<'a> InteractionCallback<'a> {
     pub(crate) const fn new(
         http: &'a Client,
-        interaction_id: InteractionId,
+        interaction_id: Id<InteractionMarker>,
         interaction_token: &'a str,
         response: &'a InteractionResponse,
     ) -> Self {
@@ -31,9 +34,21 @@ impl<'a> InteractionCallback<'a> {
         }
     }
 
-    // `self` needs to be consumed and the client returned due to parameters
-    // being consumed in request construction.
-    fn request(&self) -> Result<Request, Error> {
+    /// Execute the request, returning a future resolving to a [`Response`].
+    ///
+    /// [`Response`]: crate::response::Response
+    pub fn exec(self) -> ResponseFuture<EmptyBody> {
+        let http = self.http;
+
+        match self.try_into_request() {
+            Ok(request) => http.request(request),
+            Err(source) => ResponseFuture::error(source),
+        }
+    }
+}
+
+impl TryIntoRequest for InteractionCallback<'_> {
+    fn try_into_request(self) -> Result<Request, Error> {
         let request = Request::builder(&Route::InteractionCallback {
             interaction_id: self.interaction_id.get(),
             interaction_token: self.interaction_token,
@@ -44,41 +59,28 @@ impl<'a> InteractionCallback<'a> {
 
         Ok(request)
     }
-
-    /// Execute the request, returning a future resolving to a [`Response`].
-    ///
-    /// [`Response`]: crate::response::Response
-    pub fn exec(self) -> ResponseFuture<EmptyBody> {
-        match self.request() {
-            Ok(request) => self.http.request(request),
-            Err(source) => ResponseFuture::error(source),
-        }
-    }
 }
 
 #[cfg(test)]
 mod tests {
-    use crate::client::Client;
+    use crate::{client::Client, request::TryIntoRequest};
     use std::error::Error;
     use twilight_http_ratelimiting::Path;
-    use twilight_model::{
-        application::callback::InteractionResponse,
-        id::{ApplicationId, InteractionId},
-    };
+    use twilight_model::{application::callback::InteractionResponse, id::Id};
 
     #[test]
     fn test_interaction_callback() -> Result<(), Box<dyn Error>> {
-        let application_id = ApplicationId::new(1).expect("non zero id");
-        let interaction_id = InteractionId::new(2).expect("non zero id");
+        let application_id = Id::new(1);
+        let interaction_id = Id::new(2);
         let token = "foo".to_owned().into_boxed_str();
 
         let client = Client::new(String::new());
-        client.set_application_id(application_id);
 
         let sent_response = InteractionResponse::DeferredUpdateMessage;
         let req = client
+            .interaction(application_id)
             .interaction_callback(interaction_id, &token, &sent_response)
-            .request()?;
+            .try_into_request()?;
 
         assert!(!req.use_authorization_token());
         assert_eq!(
