@@ -31,6 +31,11 @@ mod private {
             sticker::{CreateGuildSticker, UpdateGuildSticker},
             CreateGuildChannel, CreateGuildPrune, UpdateCurrentMember, UpdateGuild,
         },
+        scheduled_event::{
+            CreateGuildExternalScheduledEvent, CreateGuildScheduledEvent,
+            CreateGuildStageInstanceScheduledEvent, CreateGuildVoiceScheduledEvent,
+            UpdateGuildScheduledEvent,
+        },
         user::UpdateCurrentUser,
     };
 
@@ -72,6 +77,11 @@ mod private {
     impl Sealed for UpdateWebhookMessage<'_> {}
     impl<'a> Sealed for UpdateCurrentUser<'a> {}
     impl Sealed for UpdateCurrentMember<'_> {}
+    impl Sealed for CreateGuildScheduledEvent<'_> {}
+    impl Sealed for CreateGuildExternalScheduledEvent<'_> {}
+    impl Sealed for CreateGuildStageInstanceScheduledEvent<'_> {}
+    impl Sealed for CreateGuildVoiceScheduledEvent<'_> {}
+    impl Sealed for UpdateGuildScheduledEvent<'_> {}
 }
 
 impl AuditLogReasonError {
