@@ -1,21 +1,25 @@
 use crate::{
     client::Client,
-    request::{self, AuditLogReason, AuditLogReasonError, Request},
+    error::Error,
+    request::{self, AuditLogReason, AuditLogReasonError, Request, TryIntoRequest},
     response::ResponseFuture,
     routing::Route,
 };
-use twilight_model::{channel::Channel, id::ChannelId};
+use twilight_model::{
+    channel::Channel,
+    id::{marker::ChannelMarker, Id},
+};
 
 /// Delete a channel by ID.
 #[must_use = "requests must be configured and executed"]
 pub struct DeleteChannel<'a> {
-    channel_id: ChannelId,
+    channel_id: Id<ChannelMarker>,
     http: &'a Client,
     reason: Option<&'a str>,
 }
 
 impl<'a> DeleteChannel<'a> {
-    pub(crate) const fn new(http: &'a Client, channel_id: ChannelId) -> Self {
+    pub(crate) const fn new(http: &'a Client, channel_id: Id<ChannelMarker>) -> Self {
         Self {
             channel_id,
             http,
@@ -27,20 +31,12 @@ impl<'a> DeleteChannel<'a> {
     ///
     /// [`Response`]: crate::response::Response
     pub fn exec(self) -> ResponseFuture<Channel> {
-        let mut request = Request::builder(&Route::DeleteChannel {
-            channel_id: self.channel_id.get(),
-        });
+        let http = self.http;
 
-        if let Some(reason) = &self.reason {
-            let header = match request::audit_header(reason) {
-                Ok(header) => header,
-                Err(source) => return ResponseFuture::error(source),
-            };
-
-            request = request.headers(header);
+        match self.try_into_request() {
+            Ok(request) => http.request(request),
+            Err(source) => ResponseFuture::error(source),
         }
-
-        self.http.request(request.build())
     }
 }
 
@@ -51,3 +47,19 @@ impl<'a> AuditLogReason<'a> for DeleteChannel<'a> {
         Ok(self)
     }
 }
+
+impl TryIntoRequest for DeleteChannel<'_> {
+    fn try_into_request(self) -> Result<Request, Error> {
+        let mut request = Request::builder(&Route::DeleteChannel {
+            channel_id: self.channel_id.get(),
+        });
+
+        if let Some(reason) = &self.reason {
+            let header = request::audit_header(reason)?;
+
+            request = request.headers(header);
+        }
+
+        Ok(request.build())
+    }
+}
