@@ -1,66 +1,22 @@
 use crate::{
     client::Client,
-    request::{self, validate_inner, AuditLogReason, AuditLogReasonError, Request},
+    error::Error as HttpError,
+    request::{self, AuditLogReason, AuditLogReasonError, Request, TryIntoRequest},
     response::ResponseFuture,
     routing::Route,
 };
 use serde::Serialize;
-use std::{
-    error::Error,
-    fmt::{Display, Formatter, Result as FmtResult},
-};
 use twilight_model::{
-    id::{ApplicationId, ChannelId, UserId},
+    id::{
+        marker::{ApplicationMarker, ChannelMarker, UserMarker},
+        Id,
+    },
     invite::{Invite, TargetType},
 };
-
-/// Error created when an invite can not be created as configured.
-#[derive(Debug)]
-pub struct CreateInviteError {
-    kind: CreateInviteErrorType,
-}
-
-impl CreateInviteError {
-    /// Immutable reference to the type of error that occurred.
-    #[must_use = "retrieving the type has no effect if left unused"]
-    pub const fn kind(&self) -> &CreateInviteErrorType {
-        &self.kind
-    }
-
-    /// Consume the error, returning the source error if there is any.
-    #[allow(clippy::unused_self)]
-    #[must_use = "consuming the error and retrieving the source has no effect if left unused"]
-    pub fn into_source(self) -> Option<Box<dyn Error + Send + Sync>> {
-        None
-    }
-
-    /// Consume the error, returning the owned error type and the source error.
-    #[must_use = "consuming the error into its parts has no effect if left unused"]
-    pub fn into_parts(self) -> (CreateInviteErrorType, Option<Box<dyn Error + Send + Sync>>) {
-        (self.kind, None)
-    }
-}
-
-impl Display for CreateInviteError {
-    fn fmt(&self, f: &mut Formatter<'_>) -> FmtResult {
-        match &self.kind {
-            CreateInviteErrorType::MaxAgeTooOld { .. } => f.write_str("max age is too long"),
-            CreateInviteErrorType::MaxUsesTooLarge { .. } => f.write_str("max uses is too many"),
-        }
-    }
-}
-
-impl Error for CreateInviteError {}
-
-/// Type of [`CreateInviteError`] that occurred.
-#[derive(Debug)]
-#[non_exhaustive]
-pub enum CreateInviteErrorType {
-    /// Configured maximum age is over 604800.
-    MaxAgeTooOld,
-    /// Configured maximum uses is over 100.
-    MaxUsesTooLarge,
-}
+use twilight_validate::request::{
+    invite_max_age as validate_invite_max_age, invite_max_uses as validate_invite_max_uses,
+    ValidationError,
+};
 
 #[derive(Serialize)]
 struct CreateInviteFields {
@@ -71,9 +27,9 @@ struct CreateInviteFields {
     #[serde(skip_serializing_if = "Option::is_none")]
     temporary: Option<bool>,
     #[serde(skip_serializing_if = "Option::is_none")]
-    target_application_id: Option<ApplicationId>,
+    target_application_id: Option<Id<ApplicationMarker>>,
     #[serde(skip_serializing_if = "Option::is_none")]
-    target_user_id: Option<UserId>,
+    target_user_id: Option<Id<UserMarker>>,
     #[serde(skip_serializing_if = "Option::is_none")]
     target_type: Option<TargetType>,
     #[serde(skip_serializing_if = "Option::is_none")]
@@ -88,13 +44,13 @@ struct CreateInviteFields {
 ///
 /// ```no_run
 /// use twilight_http::Client;
-/// use twilight_model::id::ChannelId;
+/// use twilight_model::id::Id;
 ///
 /// # #[tokio::main]
 /// # async fn main() -> Result<(), Box<dyn std::error::Error>> {
 /// let client = Client::new("my token".to_owned());
 ///
-/// let channel_id = ChannelId::new(123).expect("non zero");
+/// let channel_id = Id::new(123);
 /// let invite = client
 ///     .create_invite(channel_id)
 ///     .max_uses(3)?
@@ -106,14 +62,14 @@ struct CreateInviteFields {
 /// [`CREATE_INVITE`]: twilight_model::guild::Permissions::CREATE_INVITE
 #[must_use = "requests must be configured and executed"]
 pub struct CreateInvite<'a> {
-    channel_id: ChannelId,
+    channel_id: Id<ChannelMarker>,
     fields: CreateInviteFields,
     http: &'a Client,
     reason: Option<&'a str>,
 }
 
 impl<'a> CreateInvite<'a> {
-    pub(crate) const fn new(http: &'a Client, channel_id: ChannelId) -> Self {
+    pub(crate) const fn new(http: &'a Client, channel_id: Id<ChannelMarker>) -> Self {
         Self {
             channel_id,
             fields: CreateInviteFields {
@@ -143,12 +99,12 @@ impl<'a> CreateInvite<'a> {
     /// ```no_run
     /// use std::env;
     /// use twilight_http::Client;
-    /// use twilight_model::id::ChannelId;
+    /// use twilight_model::id::Id;
     ///
     /// # #[tokio::main]
     /// # async fn main() -> Result<(), Box<dyn std::error::Error>> {
     /// let client = Client::new(env::var("DISCORD_TOKEN")?);
-    /// let invite = client.create_invite(ChannelId::new(1).expect("non zero"))
+    /// let invite = client.create_invite(Id::new(1))
     ///     .max_age(60 * 60)?
     ///     .exec()
     ///     .await?
@@ -158,11 +114,9 @@ impl<'a> CreateInvite<'a> {
     /// println!("invite code: {}", invite.code);
     /// # Ok(()) }
     /// ```
-    pub const fn max_age(mut self, max_age: u64) -> Result<Self, CreateInviteError> {
-        if !validate_inner::invite_max_age(max_age) {
-            return Err(CreateInviteError {
-                kind: CreateInviteErrorType::MaxAgeTooOld,
-            });
+    pub const fn max_age(mut self, max_age: u64) -> Result<Self, ValidationError> {
+        if let Err(source) = validate_invite_max_age(max_age) {
+            return Err(source);
         }
 
         self.fields.max_age = Some(max_age);
@@ -181,12 +135,12 @@ impl<'a> CreateInvite<'a> {
     /// ```no_run
     /// use std::env;
     /// use twilight_http::Client;
-    /// use twilight_model::id::ChannelId;
+    /// use twilight_model::id::Id;
     ///
     /// # #[tokio::main]
     /// # async fn main() -> Result<(), Box<dyn std::error::Error>> {
     /// let client = Client::new(env::var("DISCORD_TOKEN")?);
-    /// let invite = client.create_invite(ChannelId::new(1).expect("non zero"))
+    /// let invite = client.create_invite(Id::new(1))
     ///     .max_uses(5)?
     ///     .exec()
     ///     .await?
@@ -196,11 +150,9 @@ impl<'a> CreateInvite<'a> {
     /// println!("invite code: {}", invite.code);
     /// # Ok(()) }
     /// ```
-    pub const fn max_uses(mut self, max_uses: u64) -> Result<Self, CreateInviteError> {
-        if !validate_inner::invite_max_uses(max_uses) {
-            return Err(CreateInviteError {
-                kind: CreateInviteErrorType::MaxUsesTooLarge,
-            });
+    pub const fn max_uses(mut self, max_uses: u64) -> Result<Self, ValidationError> {
+        if let Err(source) = validate_invite_max_uses(max_uses) {
+            return Err(source);
         }
 
         self.fields.max_uses = Some(max_uses);
@@ -213,14 +165,17 @@ impl<'a> CreateInvite<'a> {
     /// This only works if [`target_type`] is set to [`TargetType::EmbeddedApplication`].
     ///
     /// [`target_type`]: Self::target_type
-    pub const fn target_application_id(mut self, target_application_id: ApplicationId) -> Self {
+    pub const fn target_application_id(
+        mut self,
+        target_application_id: Id<ApplicationMarker>,
+    ) -> Self {
         self.fields.target_application_id = Some(target_application_id);
 
         self
     }
 
     /// Set the target user id for this invite.
-    pub const fn target_user_id(mut self, target_user_id: UserId) -> Self {
+    pub const fn target_user_id(mut self, target_user_id: Id<UserMarker>) -> Self {
         self.fields.target_user_id = Some(target_user_id);
 
         self
@@ -244,10 +199,10 @@ impl<'a> CreateInvite<'a> {
 
     /// Specify true if the invite should be unique. Defaults to false.
     ///
-    /// If true, don't try to reuse a similar invite (useful for creating many unique one time use
-    /// invites). Refer to [the discord docs] for more information.
+    /// If true, don't try to reuse a similar invite (useful for creating many
+    /// unique one time use invites). See [Discord Docs/Create Channel Invite].
     ///
-    /// [the discord docs]: https://discord.com/developers/docs/resources/channel#create-channel-invite
+    /// [Discord Docs/Create Channel Invite]: https://discord.com/developers/docs/resources/channel#create-channel-invite
     pub const fn unique(mut self, unique: bool) -> Self {
         self.fields.unique = Some(unique);
 
@@ -258,25 +213,12 @@ impl<'a> CreateInvite<'a> {
     ///
     /// [`Response`]: crate::response::Response
     pub fn exec(self) -> ResponseFuture<Invite> {
-        let mut request = Request::builder(&Route::CreateInvite {
-            channel_id: self.channel_id.get(),
-        });
+        let http = self.http;
 
-        request = match request.json(&self.fields) {
-            Ok(request) => request,
-            Err(source) => return ResponseFuture::error(source),
-        };
-
-        if let Some(reason) = &self.reason {
-            let header = match request::audit_header(reason) {
-                Ok(header) => header,
-                Err(source) => return ResponseFuture::error(source),
-            };
-
-            request = request.headers(header);
+        match self.try_into_request() {
+            Ok(request) => http.request(request),
+            Err(source) => ResponseFuture::error(source),
         }
-
-        self.http.request(request.build())
     }
 }
 
@@ -288,18 +230,35 @@ impl<'a> AuditLogReason<'a> for CreateInvite<'a> {
     }
 }
 
+impl TryIntoRequest for CreateInvite<'_> {
+    fn try_into_request(self) -> Result<Request, HttpError> {
+        let mut request = Request::builder(&Route::CreateInvite {
+            channel_id: self.channel_id.get(),
+        });
+
+        request = request.json(&self.fields)?;
+
+        if let Some(reason) = self.reason {
+            let header = request::audit_header(reason)?;
+
+            request = request.headers(header);
+        }
+
+        Ok(request.build())
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::CreateInvite;
     use crate::Client;
     use std::error::Error;
-    use twilight_model::id::ChannelId;
+    use twilight_model::id::Id;
 
     #[test]
     fn test_max_age() -> Result<(), Box<dyn Error>> {
         let client = Client::new("foo".to_owned());
-        let mut builder =
-            CreateInvite::new(&client, ChannelId::new(1).expect("non zero")).max_age(0)?;
+        let mut builder = CreateInvite::new(&client, Id::new(1)).max_age(0)?;
         assert_eq!(Some(0), builder.fields.max_age);
         builder = builder.max_age(604_800)?;
         assert_eq!(Some(604_800), builder.fields.max_age);
@@ -311,8 +270,7 @@ mod tests {
     #[test]
     fn test_max_uses() -> Result<(), Box<dyn Error>> {
         let client = Client::new("foo".to_owned());
-        let mut builder =
-            CreateInvite::new(&client, ChannelId::new(1).expect("non zero")).max_uses(0)?;
+        let mut builder = CreateInvite::new(&client, Id::new(1)).max_uses(0)?;
         assert_eq!(Some(0), builder.fields.max_uses);
         builder = builder.max_uses(100)?;
         assert_eq!(Some(100), builder.fields.max_uses);
