@@ -3,7 +3,6 @@ pub mod message;
 pub mod reaction;
 pub mod stage;
 pub mod thread;
-pub mod update_channel;
 pub mod webhook;
 
 mod create_pin;
@@ -15,6 +14,7 @@ mod delete_pin;
 mod follow_news_channel;
 mod get_channel;
 mod get_pins;
+mod update_channel;
 mod update_channel_permission;
 mod update_channel_permission_configured;
 
