@@ -3,7 +3,10 @@ use crate::client::Client;
 use twilight_model::{
     channel::permission_overwrite::PermissionOverwriteType,
     guild::Permissions,
-    id::{ChannelId, RoleId, UserId},
+    id::{
+        marker::{ChannelMarker, RoleMarker, UserMarker},
+        Id,
+    },
 };
 
 /// Update the permissions for a role or a user in a channel.
@@ -15,16 +18,16 @@ use twilight_model::{
 /// ```no_run
 /// use twilight_http::Client;
 /// use twilight_model::guild::Permissions;
-/// use twilight_model::id::{ChannelId, RoleId};
+/// use twilight_model::id::Id;
 ///
 /// # #[tokio::main]
 /// # async fn main() -> Result<(), Box<dyn std::error::Error>> {
 /// let client = Client::new("my token".to_owned());
 ///
-/// let channel_id = ChannelId::new(123).expect("non zero");
+/// let channel_id = Id::new(123);
 /// let allow = Permissions::VIEW_CHANNEL;
 /// let deny = Permissions::SEND_MESSAGES;
-/// let role_id = RoleId::new(432).expect("non zero");
+/// let role_id = Id::new(432);
 ///
 /// client.update_channel_permission(channel_id, allow, deny)
 ///     .role(role_id)
@@ -35,7 +38,7 @@ use twilight_model::{
 #[must_use = "requests must be configured and executed"]
 pub struct UpdateChannelPermission<'a> {
     allow: Permissions,
-    channel_id: ChannelId,
+    channel_id: Id<ChannelMarker>,
     deny: Permissions,
     http: &'a Client,
 }
@@ -43,7 +46,7 @@ pub struct UpdateChannelPermission<'a> {
 impl<'a> UpdateChannelPermission<'a> {
     pub(crate) const fn new(
         http: &'a Client,
-        channel_id: ChannelId,
+        channel_id: Id<ChannelMarker>,
         allow: Permissions,
         deny: Permissions,
     ) -> Self {
@@ -56,12 +59,12 @@ impl<'a> UpdateChannelPermission<'a> {
     }
 
     /// Specify this override to be for a member.
-    pub const fn member(self, user_id: UserId) -> UpdateChannelPermissionConfigured<'a> {
+    pub const fn member(self, user_id: Id<UserMarker>) -> UpdateChannelPermissionConfigured<'a> {
         self.configure(PermissionOverwriteType::Member(user_id))
     }
 
     /// Specify this override to be for a role.
-    pub const fn role(self, role_id: RoleId) -> UpdateChannelPermissionConfigured<'a> {
+    pub const fn role(self, role_id: Id<RoleMarker>) -> UpdateChannelPermissionConfigured<'a> {
         self.configure(PermissionOverwriteType::Role(role_id))
     }
 
