@@ -1,7 +1,7 @@
 use crate::{
     client::Client,
     error::Error,
-    request::{self, AuditLogReason, AuditLogReasonError, Request},
+    request::{self, AuditLogReason, AuditLogReasonError, Request, TryIntoRequest},
     response::{marker::EmptyBody, ResponseFuture},
     routing::Route,
 };
@@ -9,7 +9,7 @@ use serde::Serialize;
 use twilight_model::{
     channel::permission_overwrite::{PermissionOverwriteTargetType, PermissionOverwriteType},
     guild::Permissions,
-    id::ChannelId,
+    id::{marker::ChannelMarker, Id},
 };
 
 #[derive(Serialize)]
@@ -23,7 +23,7 @@ struct UpdateChannelPermissionConfiguredFields {
 /// Created when either `member` or `role` is called on a `DeleteChannelPermission` struct.
 #[must_use = "requests must be configured and executed"]
 pub struct UpdateChannelPermissionConfigured<'a> {
-    channel_id: ChannelId,
+    channel_id: Id<ChannelMarker>,
     fields: UpdateChannelPermissionConfiguredFields,
     http: &'a Client,
     target_id: u64,
@@ -33,7 +33,7 @@ pub struct UpdateChannelPermissionConfigured<'a> {
 impl<'a> UpdateChannelPermissionConfigured<'a> {
     pub(crate) const fn new(
         http: &'a Client,
-        channel_id: ChannelId,
+        channel_id: Id<ChannelMarker>,
         allow: Permissions,
         deny: Permissions,
         target: PermissionOverwriteType,
@@ -60,26 +60,14 @@ impl<'a> UpdateChannelPermissionConfigured<'a> {
         }
     }
 
-    fn request(&self) -> Result<Request, Error> {
-        let mut request = Request::builder(&Route::UpdatePermissionOverwrite {
-            channel_id: self.channel_id.get(),
-            target_id: self.target_id,
-        })
-        .json(&self.fields)?;
-
-        if let Some(reason) = &self.reason {
-            request = request.headers(request::audit_header(reason)?);
-        }
-
-        Ok(request.build())
-    }
-
     /// Execute the request, returning a future resolving to a [`Response`].
     ///
     /// [`Response`]: crate::response::Response
     pub fn exec(self) -> ResponseFuture<EmptyBody> {
-        match self.request() {
-            Ok(request) => self.http.request(request),
+        let http = self.http;
+
+        match self.try_into_request() {
+            Ok(request) => http.request(request),
             Err(source) => ResponseFuture::error(source),
         }
     }
@@ -93,14 +81,34 @@ impl<'a> AuditLogReason<'a> for UpdateChannelPermissionConfigured<'a> {
     }
 }
 
+impl TryIntoRequest for UpdateChannelPermissionConfigured<'_> {
+    fn try_into_request(self) -> Result<Request, Error> {
+        let mut request = Request::builder(&Route::UpdatePermissionOverwrite {
+            channel_id: self.channel_id.get(),
+            target_id: self.target_id,
+        })
+        .json(&self.fields)?;
+
+        if let Some(reason) = &self.reason {
+            request = request.headers(request::audit_header(reason)?);
+        }
+
+        Ok(request.build())
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::{UpdateChannelPermissionConfigured, UpdateChannelPermissionConfiguredFields};
-    use crate::{request::Request, routing::Route, Client};
+    use crate::{
+        request::{Request, TryIntoRequest},
+        routing::Route,
+        Client,
+    };
     use twilight_model::{
         channel::permission_overwrite::{PermissionOverwriteTargetType, PermissionOverwriteType},
         guild::Permissions,
-        id::{ChannelId, UserId},
+        id::Id,
     };
 
     #[test]
@@ -108,12 +116,14 @@ mod tests {
         let client = Client::new("foo".to_owned());
         let builder = UpdateChannelPermissionConfigured::new(
             &client,
-            ChannelId::new(1).expect("non zero"),
+            Id::new(1),
             Permissions::empty(),
             Permissions::SEND_MESSAGES,
-            PermissionOverwriteType::Member(UserId::new(2).expect("non zero")),
+            PermissionOverwriteType::Member(Id::new(2)),
         );
-        let actual = builder.request().expect("failed to create request");
+        let actual = builder
+            .try_into_request()
+            .expect("failed to create request");
 
         let body = crate::json::to_vec(&UpdateChannelPermissionConfiguredFields {
             allow: Permissions::empty(),
