@@ -1,11 +1,14 @@
 use crate::{
     client::Client,
     error::Error,
-    request::{self, AuditLogReason, AuditLogReasonError, Request},
+    request::{self, AuditLogReason, AuditLogReasonError, Request, TryIntoRequest},
     response::{marker::EmptyBody, ResponseFuture},
     routing::Route,
 };
-use twilight_model::id::{ChannelId, MessageId, WebhookId};
+use twilight_model::id::{
+    marker::{ChannelMarker, MessageMarker, WebhookMarker},
+    Id,
+};
 
 /// Delete a message created by a webhook.
 ///
@@ -14,13 +17,13 @@ use twilight_model::id::{ChannelId, MessageId, WebhookId};
 /// ```no_run
 /// # use twilight_http::Client;
 /// use twilight_http::request::AuditLogReason;
-/// use twilight_model::id::{MessageId, WebhookId};
+/// use twilight_model::id::Id;
 ///
 /// # #[tokio::main]
 /// # async fn main() -> Result<(), Box<dyn std::error::Error>> {
 /// # let client = Client::new("token".to_owned());
 /// client
-///     .delete_webhook_message(WebhookId::new(1).expect("non zero"), "token here", MessageId::new(2).expect("non zero"))
+///     .delete_webhook_message(Id::new(1), "token here", Id::new(2))
 ///     .reason("reason here")?
 ///     .exec()
 ///     .await?;
@@ -29,19 +32,19 @@ use twilight_model::id::{ChannelId, MessageId, WebhookId};
 #[must_use = "requests must be configured and executed"]
 pub struct DeleteWebhookMessage<'a> {
     http: &'a Client,
-    message_id: MessageId,
+    message_id: Id<MessageMarker>,
     reason: Option<&'a str>,
-    thread_id: Option<ChannelId>,
+    thread_id: Option<Id<ChannelMarker>>,
     token: &'a str,
-    webhook_id: WebhookId,
+    webhook_id: Id<WebhookMarker>,
 }
 
 impl<'a> DeleteWebhookMessage<'a> {
     pub(crate) const fn new(
         http: &'a Client,
-        webhook_id: WebhookId,
+        webhook_id: Id<WebhookMarker>,
         token: &'a str,
-        message_id: MessageId,
+        message_id: Id<MessageMarker>,
     ) -> Self {
         Self {
             http,
@@ -53,27 +56,9 @@ impl<'a> DeleteWebhookMessage<'a> {
         }
     }
 
-    // `self` needs to be consumed and the client returned due to parameters
-    // being consumed in request construction.
-    fn request(&self) -> Result<Request, Error> {
-        let mut request = Request::builder(&Route::DeleteWebhookMessage {
-            message_id: self.message_id.get(),
-            thread_id: self.thread_id.map(ChannelId::get),
-            token: self.token,
-            webhook_id: self.webhook_id.get(),
-        })
-        .use_authorization_token(false);
-
-        if let Some(reason) = self.reason.as_ref() {
-            request = request.headers(request::audit_header(reason)?);
-        }
-
-        Ok(request.build())
-    }
-
     /// Delete in a thread belonging to the channel instead of the channel
     /// itself.
-    pub fn thread_id(mut self, thread_id: ChannelId) -> Self {
+    pub fn thread_id(mut self, thread_id: Id<ChannelMarker>) -> Self {
         self.thread_id.replace(thread_id);
 
         self
@@ -83,8 +68,10 @@ impl<'a> DeleteWebhookMessage<'a> {
     ///
     /// [`Response`]: crate::response::Response
     pub fn exec(self) -> ResponseFuture<EmptyBody> {
-        match self.request() {
-            Ok(request) => self.http.request(request),
+        let http = self.http;
+
+        match self.try_into_request() {
+            Ok(request) => http.request(request),
             Err(source) => ResponseFuture::error(source),
         }
     }
@@ -98,22 +85,41 @@ impl<'a> AuditLogReason<'a> for DeleteWebhookMessage<'a> {
     }
 }
 
+impl TryIntoRequest for DeleteWebhookMessage<'_> {
+    fn try_into_request(self) -> Result<Request, Error> {
+        let mut request = Request::builder(&Route::DeleteWebhookMessage {
+            message_id: self.message_id.get(),
+            thread_id: self.thread_id.map(Id::get),
+            token: self.token,
+            webhook_id: self.webhook_id.get(),
+        })
+        .use_authorization_token(false);
+
+        if let Some(reason) = self.reason.as_ref() {
+            request = request.headers(request::audit_header(reason)?);
+        }
+
+        Ok(request.build())
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::DeleteWebhookMessage;
-    use crate::{client::Client, request::Request, routing::Route};
-    use twilight_model::id::{MessageId, WebhookId};
+    use crate::{
+        client::Client,
+        request::{Request, TryIntoRequest},
+        routing::Route,
+    };
+    use twilight_model::id::Id;
 
     #[test]
     fn test_request() {
         let client = Client::new("token".to_owned());
-        let builder = DeleteWebhookMessage::new(
-            &client,
-            WebhookId::new(1).expect("non zero"),
-            "token",
-            MessageId::new(2).expect("non zero"),
-        );
-        let actual = builder.request().expect("failed to create request");
+        let builder = DeleteWebhookMessage::new(&client, Id::new(1), "token", Id::new(2));
+        let actual = builder
+            .try_into_request()
+            .expect("failed to create request");
 
         let expected = Request::from_route(&Route::DeleteWebhookMessage {
             message_id: 2,
