@@ -1,13 +1,17 @@
 use crate::{
     client::Client,
-    request::{self, AuditLogReason, AuditLogReasonError, NullableField, Request},
+    error::Error,
+    request::{self, AuditLogReason, AuditLogReasonError, NullableField, Request, TryIntoRequest},
     response::ResponseFuture,
     routing::Route,
 };
 use serde::Serialize;
 use twilight_model::{
     channel::Webhook,
-    id::{ChannelId, WebhookId},
+    id::{
+        marker::{ChannelMarker, WebhookMarker},
+        Id,
+    },
 };
 
 #[derive(Serialize)]
@@ -15,7 +19,7 @@ struct UpdateWebhookFields<'a> {
     #[serde(skip_serializing_if = "Option::is_none")]
     avatar: Option<NullableField<&'a str>>,
     #[serde(skip_serializing_if = "Option::is_none")]
-    channel_id: Option<ChannelId>,
+    channel_id: Option<Id<ChannelMarker>>,
     #[serde(skip_serializing_if = "Option::is_none")]
     name: Option<NullableField<&'a str>>,
 }
@@ -25,13 +29,13 @@ struct UpdateWebhookFields<'a> {
 pub struct UpdateWebhook<'a> {
     fields: UpdateWebhookFields<'a>,
     http: &'a Client,
-    webhook_id: WebhookId,
+    webhook_id: Id<WebhookMarker>,
     reason: Option<&'a str>,
 }
 
 /// Update a webhook by its ID.
 impl<'a> UpdateWebhook<'a> {
-    pub(crate) const fn new(http: &'a Client, webhook_id: WebhookId) -> Self {
+    pub(crate) const fn new(http: &'a Client, webhook_id: Id<WebhookMarker>) -> Self {
         Self {
             fields: UpdateWebhookFields {
                 avatar: None,
@@ -46,9 +50,9 @@ impl<'a> UpdateWebhook<'a> {
 
     /// Set the avatar of the webhook.
     ///
-    /// See [Discord Docs/Image Data] for more information. This must be a Data URI, in the form of
-    /// `data:image/{type};base64,{data}` where `{type}` is the image MIME type and `{data}` is the
-    /// base64-encoded image.
+    /// This must be a Data URI, in the form of
+    /// `data:image/{type};base64,{data}` where `{type}` is the image MIME type
+    /// and `{data}` is the base64-encoded image. See [Discord Docs/Image Data].
     ///
     /// [Discord Docs/Image Data]: https://discord.com/developers/docs/reference#image-data
     pub const fn avatar(mut self, avatar: Option<&'a str>) -> Self {
@@ -58,7 +62,7 @@ impl<'a> UpdateWebhook<'a> {
     }
 
     /// Move this webhook to a new channel.
-    pub const fn channel_id(mut self, channel_id: ChannelId) -> Self {
+    pub const fn channel_id(mut self, channel_id: Id<ChannelMarker>) -> Self {
         self.fields.channel_id = Some(channel_id);
 
         self
@@ -75,26 +79,12 @@ impl<'a> UpdateWebhook<'a> {
     ///
     /// [`Response`]: crate::response::Response
     pub fn exec(self) -> ResponseFuture<Webhook> {
-        let mut request = Request::builder(&Route::UpdateWebhook {
-            token: None,
-            webhook_id: self.webhook_id.get(),
-        });
+        let http = self.http;
 
-        request = match request.json(&self.fields) {
-            Ok(request) => request,
-            Err(source) => return ResponseFuture::error(source),
-        };
-
-        if let Some(reason) = self.reason.as_ref() {
-            let header = match request::audit_header(reason) {
-                Ok(header) => header,
-                Err(source) => return ResponseFuture::error(source),
-            };
-
-            request = request.headers(header);
+        match self.try_into_request() {
+            Ok(request) => http.request(request),
+            Err(source) => ResponseFuture::error(source),
         }
-
-        self.http.request(request.build())
     }
 }
 
@@ -105,3 +95,22 @@ impl<'a> AuditLogReason<'a> for UpdateWebhook<'a> {
         Ok(self)
     }
 }
+
+impl TryIntoRequest for UpdateWebhook<'_> {
+    fn try_into_request(self) -> Result<Request, Error> {
+        let mut request = Request::builder(&Route::UpdateWebhook {
+            token: None,
+            webhook_id: self.webhook_id.get(),
+        });
+
+        request = request.json(&self.fields)?;
+
+        if let Some(reason) = self.reason.as_ref() {
+            let header = request::audit_header(reason)?;
+
+            request = request.headers(header);
+        }
+
+        Ok(request.build())
+    }
+}
