@@ -4,127 +4,33 @@ use crate::{
     client::Client,
     error::Error as HttpError,
     request::{
-        self,
-        validate_inner::{self, ComponentValidationError, ComponentValidationErrorType},
-        AttachmentFile, AuditLogReason, AuditLogReasonError, Form, NullableField, Request,
+        self, AttachmentFile, AuditLogReason, AuditLogReasonError, Form, NullableField, Request,
+        TryIntoRequest,
     },
     response::{marker::EmptyBody, ResponseFuture},
     routing::Route,
 };
 use serde::Serialize;
-use std::{
-    borrow::Cow,
-    error::Error,
-    fmt::{Display, Formatter, Result as FmtResult},
-};
+use std::borrow::Cow;
 use twilight_model::{
     application::component::Component,
     channel::{embed::Embed, message::AllowedMentions, Attachment},
-    id::{ChannelId, MessageId, WebhookId},
+    id::{
+        marker::{ChannelMarker, MessageMarker, WebhookMarker},
+        Id,
+    },
+};
+use twilight_validate::message::{
+    components as validate_components, content as validate_content, embeds as validate_embeds,
+    MessageValidationError,
 };
-
-/// A webhook's message can not be updated as configured.
-#[derive(Debug)]
-pub struct UpdateWebhookMessageError {
-    kind: UpdateWebhookMessageErrorType,
-    source: Option<Box<dyn Error + Send + Sync>>,
-}
-
-impl UpdateWebhookMessageError {
-    /// Immutable reference to the type of error that occurred.
-    #[must_use = "retrieving the type has no effect if left unused"]
-    pub const fn kind(&self) -> &UpdateWebhookMessageErrorType {
-        &self.kind
-    }
-
-    /// Consume the error, returning the source error if there is any.
-    #[must_use = "consuming the error and retrieving the source has no effect if left unused"]
-    pub fn into_source(self) -> Option<Box<dyn Error + Send + Sync>> {
-        self.source
-    }
-
-    /// Consume the error, returning the owned error type and the source error.
-    #[must_use = "consuming the error into its parts has no effect if left unused"]
-    pub fn into_parts(
-        self,
-    ) -> (
-        UpdateWebhookMessageErrorType,
-        Option<Box<dyn Error + Send + Sync>>,
-    ) {
-        (self.kind, self.source)
-    }
-}
-
-impl Display for UpdateWebhookMessageError {
-    fn fmt(&self, f: &mut Formatter<'_>) -> FmtResult {
-        match &self.kind {
-            UpdateWebhookMessageErrorType::ComponentCount { count } => {
-                Display::fmt(count, f)?;
-                f.write_str(" components were provided, but only ")?;
-                Display::fmt(&ComponentValidationError::COMPONENT_COUNT, f)?;
-
-                f.write_str(" root components are allowed")
-            }
-            UpdateWebhookMessageErrorType::ComponentInvalid { .. } => {
-                f.write_str("a provided component is invalid")
-            }
-            UpdateWebhookMessageErrorType::ContentInvalid => {
-                f.write_str("message content is invalid")
-            }
-            UpdateWebhookMessageErrorType::EmbedTooLarge { .. } => {
-                f.write_str("length of one of the embeds is too large")
-            }
-            UpdateWebhookMessageErrorType::TooManyEmbeds => {
-                f.write_str("only 10 embeds may be provided")
-            }
-        }
-    }
-}
-
-impl Error for UpdateWebhookMessageError {
-    fn source(&self) -> Option<&(dyn Error + 'static)> {
-        self.source
-            .as_ref()
-            .map(|source| &**source as &(dyn Error + 'static))
-    }
-}
-
-/// Type of [`UpdateWebhookMessageError`] that occurred.
-#[derive(Debug)]
-#[non_exhaustive]
-pub enum UpdateWebhookMessageErrorType {
-    /// Content is over 2000 UTF-16 characters.
-    ContentInvalid,
-    /// Length of one of the embeds is over 6000 characters.
-    EmbedTooLarge {
-        /// Index of the embed that was too large.
-        ///
-        /// This can be used to index into the provided embeds to retrieve the
-        /// invalid embed.
-        index: usize,
-    },
-    /// An invalid message component was provided.
-    ComponentInvalid {
-        /// Additional details about the validation failure type.
-        kind: ComponentValidationErrorType,
-    },
-    /// Too many message components were provided.
-    ComponentCount {
-        /// Number of components that were provided.
-        count: usize,
-    },
-    /// Too many embeds were provided.
-    ///
-    /// A webhook can have up to 10 embeds.
-    TooManyEmbeds,
-}
 
 #[derive(Serialize)]
 struct UpdateWebhookMessageFields<'a> {
     #[serde(skip_serializing_if = "Option::is_none")]
     allowed_mentions: Option<AllowedMentions>,
-    #[serde(skip_serializing_if = "request::slice_is_empty")]
-    attachments: &'a [Attachment],
+    #[serde(skip_serializing_if = "Option::is_none")]
+    attachments: Option<NullableField<&'a [Attachment]>>,
     #[serde(skip_serializing_if = "Option::is_none")]
     components: Option<NullableField<&'a [Component]>>,
     #[serde(skip_serializing_if = "Option::is_none")]
@@ -151,13 +57,13 @@ struct UpdateWebhookMessageFields<'a> {
 /// # use twilight_http::Client;
 /// use twilight_model::{
 ///     channel::message::AllowedMentions,
-///     id::{MessageId, WebhookId}
+///     id::Id,
 /// };
 ///
 /// # #[tokio::main]
 /// # async fn main() -> Result<(), Box<dyn std::error::Error>> {
 /// # let client = Client::new("token".to_owned());
-/// client.update_webhook_message(WebhookId::new(1).expect("non zero"), "token here", MessageId::new(2).expect("non zero"))
+/// client.update_webhook_message(Id::new(1), "token here", Id::new(2))
 ///     // By creating a default set of allowed mentions, no entity can be
 ///     // mentioned.
 ///     .allowed_mentions(AllowedMentions::default())
@@ -173,11 +79,11 @@ pub struct UpdateWebhookMessage<'a> {
     attachments: Cow<'a, [AttachmentFile<'a>]>,
     fields: UpdateWebhookMessageFields<'a>,
     http: &'a Client,
-    message_id: MessageId,
+    message_id: Id<MessageMarker>,
     reason: Option<&'a str>,
-    thread_id: Option<ChannelId>,
+    thread_id: Option<Id<ChannelMarker>>,
     token: &'a str,
-    webhook_id: WebhookId,
+    webhook_id: Id<WebhookMarker>,
 }
 
 impl<'a> UpdateWebhookMessage<'a> {
@@ -186,14 +92,14 @@ impl<'a> UpdateWebhookMessage<'a> {
 
     pub(crate) const fn new(
         http: &'a Client,
-        webhook_id: WebhookId,
+        webhook_id: Id<WebhookMarker>,
         token: &'a str,
-        message_id: MessageId,
+        message_id: Id<MessageMarker>,
     ) -> Self {
         Self {
             fields: UpdateWebhookMessageFields {
                 allowed_mentions: None,
-                attachments: &[],
+                attachments: None,
                 components: None,
                 content: None,
                 embeds: None,
@@ -221,7 +127,7 @@ impl<'a> UpdateWebhookMessage<'a> {
     /// If called, all unspecified attachments will be removed from the message.
     /// If not called, all attachments will be kept.
     pub const fn attachments(mut self, attachments: &'a [Attachment]) -> Self {
-        self.fields.attachments = attachments;
+        self.fields.attachments = Some(NullableField(Some(attachments)));
 
         self
     }
@@ -234,32 +140,15 @@ impl<'a> UpdateWebhookMessage<'a> {
     ///
     /// # Errors
     ///
-    /// Returns an [`UpdateWebhookMessageErrorType::ComponentCount`] error
-    /// type if too many components are provided.
-    ///
-    /// Returns an [`UpdateWebhookMessageErrorType::ComponentInvalid`] error
-    /// type if one of the provided components is invalid.
+    /// Refer to the errors section of
+    /// [`twilight_validate::component::component`] for a list of errors that
+    /// may be returned as a result of validating each provided component.
     pub fn components(
         mut self,
         components: Option<&'a [Component]>,
-    ) -> Result<Self, UpdateWebhookMessageError> {
-        if let Some(components) = components.as_ref() {
-            validate_inner::components(components).map_err(|source| {
-                let (kind, inner_source) = source.into_parts();
-
-                match kind {
-                    ComponentValidationErrorType::ComponentCount { count } => {
-                        UpdateWebhookMessageError {
-                            kind: UpdateWebhookMessageErrorType::ComponentCount { count },
-                            source: inner_source,
-                        }
-                    }
-                    other => UpdateWebhookMessageError {
-                        kind: UpdateWebhookMessageErrorType::ComponentInvalid { kind: other },
-                        source: inner_source,
-                    },
-                }
-            })?;
+    ) -> Result<Self, MessageValidationError> {
+        if let Some(components) = components {
+            validate_components(components)?;
         }
 
         self.fields.components = Some(NullableField(components));
@@ -278,16 +167,13 @@ impl<'a> UpdateWebhookMessage<'a> {
     ///
     /// # Errors
     ///
-    /// Returns an [`UpdateWebhookMessageErrorType::ContentInvalid`] error type if
-    /// the content length is too long.
-    pub fn content(mut self, content: Option<&'a str>) -> Result<Self, UpdateWebhookMessageError> {
+    /// Returns an error of type [`ContentInvalid`] if the content length is too
+    /// long.
+    ///
+    /// [`ContentInvalid`]: twilight_validate::message::MessageValidationErrorType::ContentInvalid
+    pub fn content(mut self, content: Option<&'a str>) -> Result<Self, MessageValidationError> {
         if let Some(content_ref) = content {
-            if !validate_inner::content_limit(content_ref) {
-                return Err(UpdateWebhookMessageError {
-                    kind: UpdateWebhookMessageErrorType::ContentInvalid,
-                    source: None,
-                });
-            }
+            validate_content(content_ref)?;
         }
 
         self.fields.content = Some(NullableField(content));
@@ -304,7 +190,7 @@ impl<'a> UpdateWebhookMessage<'a> {
     ///
     /// The total character length of each embed must not exceed 6000
     /// characters. Additionally, the internal fields also have character
-    /// limits. Refer to [the discord docs] for more information.
+    /// limits. See [Discord Docs/Embed Limits].
     ///
     /// # Examples
     ///
@@ -315,7 +201,7 @@ impl<'a> UpdateWebhookMessage<'a> {
     /// ```no_run
     /// # use twilight_http::Client;
     /// use twilight_embed_builder::EmbedBuilder;
-    /// use twilight_model::id::{MessageId, WebhookId};
+    /// use twilight_model::id::Id;
     ///
     /// # #[tokio::main] async fn main() -> Result<(), Box<dyn std::error::Error>> {
     /// # let client = Client::new("token".to_owned());
@@ -325,7 +211,7 @@ impl<'a> UpdateWebhookMessage<'a> {
     ///     .url("https://twilight.rs")
     ///     .build()?;
     ///
-    /// client.update_webhook_message(WebhookId::new(1).expect("non zero"), "token", MessageId::new(2).expect("non zero"))
+    /// client.update_webhook_message(Id::new(1), "token", Id::new(2))
     ///     .embeds(Some(&[embed]))?
     ///     .exec()
     ///     .await?;
@@ -334,34 +220,18 @@ impl<'a> UpdateWebhookMessage<'a> {
     ///
     /// # Errors
     ///
-    /// Returns an [`UpdateWebhookMessageErrorType::EmbedTooLarge`] error type
-    /// if one of the embeds are too large.
+    /// Returns an error of type [`TooManyEmbeds`] if there are too many embeds.
     ///
-    /// Returns an [`UpdateWebhookMessageErrorType::TooManyEmbeds`] error type
-    /// if more than 10 embeds are provided.
+    /// Otherwise, refer to the errors section of [`embed`] for a list of errors
+    /// that may occur.
     ///
-    /// [the discord docs]: https://discord.com/developers/docs/resources/channel#embed-limits
-    /// [`EMBED_COUNT_LIMIT`]: Self::EMBED_COUNT_LIMIT
-    pub fn embeds(
-        mut self,
-        embeds: Option<&'a [Embed]>,
-    ) -> Result<Self, UpdateWebhookMessageError> {
-        if let Some(embeds_present) = embeds {
-            if embeds_present.len() > Self::EMBED_COUNT_LIMIT {
-                return Err(UpdateWebhookMessageError {
-                    kind: UpdateWebhookMessageErrorType::TooManyEmbeds,
-                    source: None,
-                });
-            }
-
-            for (idx, embed) in embeds_present.iter().enumerate() {
-                if let Err(source) = validate_inner::embed(embed) {
-                    return Err(UpdateWebhookMessageError {
-                        kind: UpdateWebhookMessageErrorType::EmbedTooLarge { index: idx },
-                        source: Some(Box::new(source)),
-                    });
-                }
-            }
+    /// [`EMBED_COUNT_LIMIT`]: twilight_validate::message::EMBED_COUNT_LIMIT
+    /// [`embed`]: twilight_validate::embed::embed
+    /// [`TooManyEmbeds`]: twilight_validate::message::MessageValidationErrorType::TooManyEmbeds
+    /// [Discord Docs/Embed Limits]: https://discord.com/developers/docs/resources/channel#embed-limits
+    pub fn embeds(mut self, embeds: Option<&'a [Embed]>) -> Result<Self, MessageValidationError> {
+        if let Some(embeds) = embeds {
+            validate_embeds(embeds)?;
         }
 
         self.fields.embeds = Some(NullableField(embeds));
@@ -406,18 +276,38 @@ impl<'a> UpdateWebhookMessage<'a> {
 
     /// Update in a thread belonging to the channel instead of the channel
     /// itself.
-    pub fn thread_id(mut self, thread_id: ChannelId) -> Self {
+    pub fn thread_id(mut self, thread_id: Id<ChannelMarker>) -> Self {
         self.thread_id.replace(thread_id);
 
         self
     }
 
-    // `self` needs to be consumed and the client returned due to parameters
-    // being consumed in request construction.
-    fn request(&mut self) -> Result<Request, HttpError> {
+    /// Execute the request, returning a future resolving to a [`Response`].
+    ///
+    /// [`Response`]: crate::response::Response
+    pub fn exec(self) -> ResponseFuture<EmptyBody> {
+        let http = self.http;
+
+        match self.try_into_request() {
+            Ok(request) => http.request(request),
+            Err(source) => ResponseFuture::error(source),
+        }
+    }
+}
+
+impl<'a> AuditLogReason<'a> for UpdateWebhookMessage<'a> {
+    fn reason(mut self, reason: &'a str) -> Result<Self, AuditLogReasonError> {
+        self.reason.replace(AuditLogReasonError::validate(reason)?);
+
+        Ok(self)
+    }
+}
+
+impl TryIntoRequest for UpdateWebhookMessage<'_> {
+    fn try_into_request(self) -> Result<Request, HttpError> {
         let mut request = Request::builder(&Route::UpdateWebhookMessage {
             message_id: self.message_id.get(),
-            thread_id: self.thread_id.map(ChannelId::get),
+            thread_id: self.thread_id.map(Id::get),
             token: self.token,
             webhook_id: self.webhook_id.get(),
         })
@@ -439,21 +329,25 @@ impl<'a> UpdateWebhookMessage<'a> {
             if let Some(payload_json) = &self.fields.payload_json {
                 form.payload_json(payload_json);
             } else {
-                if self.fields.allowed_mentions.is_none() {
-                    self.fields.allowed_mentions = self.http.default_allowed_mentions();
+                let mut fields = self.fields;
+
+                if fields.allowed_mentions.is_none() {
+                    fields.allowed_mentions = self.http.default_allowed_mentions();
                 }
 
-                let body = crate::json::to_vec(&self.fields).map_err(HttpError::json)?;
+                let body = crate::json::to_vec(&fields).map_err(HttpError::json)?;
                 form.payload_json(&body);
             }
 
             request = request.form(form);
         } else {
-            if self.fields.allowed_mentions.is_none() {
-                self.fields.allowed_mentions = self.http.default_allowed_mentions();
+            let mut fields = self.fields;
+
+            if fields.allowed_mentions.is_none() {
+                fields.allowed_mentions = self.http.default_allowed_mentions();
             }
 
-            request = request.json(&self.fields)?;
+            request = request.json(&fields)?;
         }
 
         if let Some(reason) = self.reason.as_ref() {
@@ -462,24 +356,6 @@ impl<'a> UpdateWebhookMessage<'a> {
 
         Ok(request.build())
     }
-
-    /// Execute the request, returning a future resolving to a [`Response`].
-    ///
-    /// [`Response`]: crate::response::Response
-    pub fn exec(mut self) -> ResponseFuture<EmptyBody> {
-        match self.request() {
-            Ok(request) => self.http.request(request),
-            Err(source) => ResponseFuture::error(source),
-        }
-    }
-}
-
-impl<'a> AuditLogReason<'a> for UpdateWebhookMessage<'a> {
-    fn reason(mut self, reason: &'a str) -> Result<Self, AuditLogReasonError> {
-        self.reason.replace(AuditLogReasonError::validate(reason)?);
-
-        Ok(self)
-    }
 }
 
 #[cfg(test)]
@@ -487,30 +363,27 @@ mod tests {
     use super::{UpdateWebhookMessage, UpdateWebhookMessageFields};
     use crate::{
         client::Client,
-        request::{AuditLogReason, NullableField, Request},
+        request::{AuditLogReason, NullableField, Request, TryIntoRequest},
         routing::Route,
     };
-    use twilight_model::id::{ChannelId, MessageId, WebhookId};
+    use twilight_model::id::Id;
 
     #[test]
     fn test_request() {
         let client = Client::new("token".to_owned());
-        let mut builder = UpdateWebhookMessage::new(
-            &client,
-            WebhookId::new(1).expect("non zero"),
-            "token",
-            MessageId::new(2).expect("non zero"),
-        )
-        .content(Some("test"))
-        .expect("'test' content couldn't be set")
-        .thread_id(ChannelId::new(3).expect("non zero"))
-        .reason("reason")
-        .expect("'reason' is not a valid reason");
-        let actual = builder.request().expect("failed to create request");
+        let builder = UpdateWebhookMessage::new(&client, Id::new(1), "token", Id::new(2))
+            .content(Some("test"))
+            .expect("'test' content couldn't be set")
+            .thread_id(Id::new(3))
+            .reason("reason")
+            .expect("'reason' is not a valid reason");
+        let actual = builder
+            .try_into_request()
+            .expect("failed to create request");
 
         let body = UpdateWebhookMessageFields {
             allowed_mentions: None,
-            attachments: &[],
+            attachments: None,
             components: None,
             content: Some(NullableField(Some("test"))),
             embeds: None,
