@@ -1,10 +1,14 @@
 use crate::{
     client::Client,
-    request::{self, AuditLogReason, AuditLogReasonError, Request},
+    error::Error,
+    request::{self, AuditLogReason, AuditLogReasonError, Request, TryIntoRequest},
     response::{marker::EmptyBody, ResponseFuture},
     routing::Route,
 };
-use twilight_model::id::{GuildId, UserId};
+use twilight_model::id::{
+    marker::{GuildMarker, UserMarker},
+    Id,
+};
 
 /// Remove a ban from a user in a guild.
 ///
@@ -14,28 +18,32 @@ use twilight_model::id::{GuildId, UserId};
 ///
 /// ```no_run
 /// use twilight_http::Client;
-/// use twilight_model::id::{GuildId, UserId};
+/// use twilight_model::id::Id;
 ///
 /// # #[tokio::main]
 /// # async fn main() -> Result<(), Box<dyn std::error::Error>> {
 /// let client = Client::new("my token".to_owned());
 ///
-/// let guild_id = GuildId::new(100).expect("non zero");
-/// let user_id = UserId::new(200).expect("non zero");
+/// let guild_id = Id::new(100);
+/// let user_id = Id::new(200);
 ///
 /// client.delete_ban(guild_id, user_id).exec().await?;
 /// # Ok(()) }
 /// ```
 #[must_use = "requests must be configured and executed"]
 pub struct DeleteBan<'a> {
-    guild_id: GuildId,
+    guild_id: Id<GuildMarker>,
     http: &'a Client,
-    user_id: UserId,
+    user_id: Id<UserMarker>,
     reason: Option<&'a str>,
 }
 
 impl<'a> DeleteBan<'a> {
-    pub(crate) const fn new(http: &'a Client, guild_id: GuildId, user_id: UserId) -> Self {
+    pub(crate) const fn new(
+        http: &'a Client,
+        guild_id: Id<GuildMarker>,
+        user_id: Id<UserMarker>,
+    ) -> Self {
         Self {
             guild_id,
             http,
@@ -48,21 +56,12 @@ impl<'a> DeleteBan<'a> {
     ///
     /// [`Response`]: crate::response::Response
     pub fn exec(self) -> ResponseFuture<EmptyBody> {
-        let mut request = Request::builder(&Route::DeleteBan {
-            guild_id: self.guild_id.get(),
-            user_id: self.user_id.get(),
-        });
+        let http = self.http;
 
-        if let Some(reason) = self.reason.as_ref() {
-            let header = match request::audit_header(reason) {
-                Ok(header) => header,
-                Err(source) => return ResponseFuture::error(source),
-            };
-
-            request = request.headers(header);
+        match self.try_into_request() {
+            Ok(request) => http.request(request),
+            Err(source) => ResponseFuture::error(source),
         }
-
-        self.http.request(request.build())
     }
 }
 
@@ -73,3 +72,20 @@ impl<'a> AuditLogReason<'a> for DeleteBan<'a> {
         Ok(self)
     }
 }
+
+impl TryIntoRequest for DeleteBan<'_> {
+    fn try_into_request(self) -> Result<Request, Error> {
+        let mut request = Request::builder(&Route::DeleteBan {
+            guild_id: self.guild_id.get(),
+            user_id: self.user_id.get(),
+        });
+
+        if let Some(reason) = self.reason.as_ref() {
+            let header = request::audit_header(reason)?;
+
+            request = request.headers(header);
+        }
+
+        Ok(request.build())
+    }
+}
