@@ -1,79 +1,22 @@
 use crate::{
     client::Client,
-    request::{self, validate_inner, AuditLogReason, AuditLogReasonError, Request},
+    error::Error as HttpError,
+    request::{self, AuditLogReason, AuditLogReasonError, Request, TryIntoRequest},
     response::ResponseFuture,
     routing::Route,
 };
 use serde::Serialize;
-use std::{
-    error::Error,
-    fmt::{Display, Formatter, Result as FmtResult},
-};
 use twilight_model::{
     channel::{permission_overwrite::PermissionOverwrite, ChannelType, GuildChannel},
-    id::{ChannelId, GuildId},
+    id::{
+        marker::{ChannelMarker, GuildMarker},
+        Id,
+    },
+};
+use twilight_validate::channel::{
+    name as validate_name, rate_limit_per_user as validate_rate_limit_per_user,
+    topic as validate_topic, ChannelValidationError,
 };
-
-/// Returned when the channel can not be created as configured.
-#[derive(Debug)]
-pub struct CreateGuildChannelError {
-    kind: CreateGuildChannelErrorType,
-}
-
-impl CreateGuildChannelError {
-    /// Immutable reference to the type of error that occurred.
-    #[must_use = "retrieving the type has no effect if left unused"]
-    pub const fn kind(&self) -> &CreateGuildChannelErrorType {
-        &self.kind
-    }
-
-    /// Consume the error, returning the source error if there is any.
-    #[allow(clippy::unused_self)]
-    #[must_use = "consuming the error and retrieving the source has no effect if left unused"]
-    pub fn into_source(self) -> Option<Box<dyn Error + Send + Sync>> {
-        None
-    }
-
-    /// Consume the error, returning the owned error type and the source error.
-    #[must_use = "consuming the error into its parts has no effect if left unused"]
-    pub fn into_parts(
-        self,
-    ) -> (
-        CreateGuildChannelErrorType,
-        Option<Box<dyn Error + Send + Sync>>,
-    ) {
-        (self.kind, None)
-    }
-}
-
-/// Type of [`CreateGuildChannelError`] that occurred.
-#[derive(Debug)]
-#[non_exhaustive]
-pub enum CreateGuildChannelErrorType {
-    /// The length of the name is either fewer than 1 UTF-16 characters or
-    /// more than 100 UTF-16 characters.
-    NameInvalid,
-    /// The seconds of the rate limit per user is more than 21600.
-    RateLimitPerUserInvalid,
-    /// The length of the topic is more than 1024 UTF-16 characters.
-    TopicInvalid,
-}
-
-impl Display for CreateGuildChannelError {
-    fn fmt(&self, f: &mut Formatter<'_>) -> FmtResult {
-        match &self.kind {
-            CreateGuildChannelErrorType::NameInvalid => {
-                f.write_str("the length of the name is invalid")
-            }
-            CreateGuildChannelErrorType::RateLimitPerUserInvalid => {
-                f.write_str("the rate limit per user is invalid")
-            }
-            CreateGuildChannelErrorType::TopicInvalid => f.write_str("the topic is invalid"),
-        }
-    }
-}
-
-impl Error for CreateGuildChannelError {}
 
 #[derive(Serialize)]
 struct CreateGuildChannelFields<'a> {
@@ -85,7 +28,7 @@ struct CreateGuildChannelFields<'a> {
     #[serde(skip_serializing_if = "Option::is_none")]
     nsfw: Option<bool>,
     #[serde(skip_serializing_if = "Option::is_none")]
-    parent_id: Option<ChannelId>,
+    parent_id: Option<Id<ChannelMarker>>,
     #[serde(skip_serializing_if = "Option::is_none")]
     permission_overwrites: Option<&'a [PermissionOverwrite]>,
     #[serde(skip_serializing_if = "Option::is_none")]
@@ -105,7 +48,7 @@ struct CreateGuildChannelFields<'a> {
 #[must_use = "requests must be configured and executed"]
 pub struct CreateGuildChannel<'a> {
     fields: CreateGuildChannelFields<'a>,
-    guild_id: GuildId,
+    guild_id: Id<GuildMarker>,
     http: &'a Client,
     reason: Option<&'a str>,
 }
@@ -113,14 +56,10 @@ pub struct CreateGuildChannel<'a> {
 impl<'a> CreateGuildChannel<'a> {
     pub(crate) fn new(
         http: &'a Client,
-        guild_id: GuildId,
+        guild_id: Id<GuildMarker>,
         name: &'a str,
-    ) -> Result<Self, CreateGuildChannelError> {
-        if !validate_inner::channel_name(name) {
-            return Err(CreateGuildChannelError {
-                kind: CreateGuildChannelErrorType::NameInvalid,
-            });
-        }
+    ) -> Result<Self, ChannelValidationError> {
+        validate_name(name)?;
 
         Ok(Self {
             fields: CreateGuildChannelFields {
@@ -164,7 +103,7 @@ impl<'a> CreateGuildChannel<'a> {
 
     /// If this is specified, and the parent ID is a `ChannelType::CategoryChannel`, create this
     /// channel as a child of the category channel.
-    pub const fn parent_id(mut self, parent_id: ChannelId) -> Self {
+    pub const fn parent_id(mut self, parent_id: Id<ChannelMarker>) -> Self {
         self.fields.parent_id = Some(parent_id);
 
         self
@@ -193,23 +132,22 @@ impl<'a> CreateGuildChannel<'a> {
     /// Set the number of seconds that a user must wait before before they are able to send another
     /// message.
     ///
-    /// The minimum is 0 and the maximum is 21600. Refer to [the discord docs] for more details.
-    /// This is also known as "Slow Mode".
+    /// The minimum is 0 and the maximum is 21600. This is also known as "Slow
+    /// Mode". See [Discord Docs/Channel Object].
     ///
     /// # Errors
     ///
-    /// Returns a [`CreateGuildChannelErrorType::RateLimitPerUserInvalid`] error
-    /// type if the amount is greater than 21600.
+    /// Returns an error of type [`RateLimitPerUserInvalid`] if the name is
+    /// invalid.
     ///
-    /// [the discord docs]: https://discordapp.com/developers/docs/resources/channel#channel-object-channel-structure
+    /// [`RateLimitPerUserInvalid`]: twilight_validate::channel::ChannelValidationErrorType::RateLimitPerUserInvalid
+    /// [Discord Docs/Channel Object]: https://discordapp.com/developers/docs/resources/channel#channel-object-channel-structure
     pub const fn rate_limit_per_user(
         mut self,
         rate_limit_per_user: u64,
-    ) -> Result<Self, CreateGuildChannelError> {
-        if rate_limit_per_user > 21600 {
-            return Err(CreateGuildChannelError {
-                kind: CreateGuildChannelErrorType::RateLimitPerUserInvalid,
-            });
+    ) -> Result<Self, ChannelValidationError> {
+        if let Err(source) = validate_rate_limit_per_user(rate_limit_per_user) {
+            return Err(source);
         }
 
         self.fields.rate_limit_per_user = Some(rate_limit_per_user);
@@ -219,20 +157,18 @@ impl<'a> CreateGuildChannel<'a> {
 
     /// Set the topic.
     ///
-    /// The maximum length is 1024 UTF-16 characters. Refer to [the discord docs] for more details.
+    /// The maximum length is 1024 UTF-16 characters. See
+    /// [Discord Docs/Channel Object].
     ///
     /// # Errors
     ///
-    /// Returns a [`CreateGuildChannelErrorType::TopicInvalid`] error type if
-    /// the topic length is too long.
+    /// Returns an error of type [`TopicInvalid`] if the name is
+    /// invalid.
     ///
-    /// [the discord docs]: https://discordapp.com/developers/docs/resources/channel#channel-object-channel-structure
-    pub fn topic(mut self, topic: &'a str) -> Result<Self, CreateGuildChannelError> {
-        if topic.chars().count() > 1024 {
-            return Err(CreateGuildChannelError {
-                kind: CreateGuildChannelErrorType::TopicInvalid,
-            });
-        }
+    /// [`TopicInvalid`]: twilight_validate::channel::ChannelValidationErrorType::TopicInvalid
+    /// [Discord Docs/Channel Object]: https://discordapp.com/developers/docs/resources/channel#channel-object-channel-structure
+    pub fn topic(mut self, topic: &'a str) -> Result<Self, ChannelValidationError> {
+        validate_topic(topic)?;
 
         self.fields.topic.replace(topic);
 
@@ -241,10 +177,10 @@ impl<'a> CreateGuildChannel<'a> {
 
     /// For voice channels, set the user limit.
     ///
-    /// Set to 0 for no limit. Limit can otherwise be between 1 and 99 inclusive. Refer to [the
-    /// discord docs] for more details.
+    /// Set to 0 for no limit. Limit can otherwise be between 1 and 99
+    /// inclusive. See [Discord Docs/Modify Channel] for more details.
     ///
-    /// [the discord docs]: https://discord.com/developers/docs/resources/channel#modify-channel-json-params
+    /// [Discord Docs/Modify Channel]: https://discord.com/developers/docs/resources/channel#modify-channel-json-params-guild-channel
     pub const fn user_limit(mut self, user_limit: u64) -> Self {
         self.fields.user_limit = Some(user_limit);
 
@@ -255,25 +191,12 @@ impl<'a> CreateGuildChannel<'a> {
     ///
     /// [`Response`]: crate::response::Response
     pub fn exec(self) -> ResponseFuture<GuildChannel> {
-        let mut request = Request::builder(&Route::CreateChannel {
-            guild_id: self.guild_id.get(),
-        });
+        let http = self.http;
 
-        request = match request.json(&self.fields) {
-            Ok(request) => request,
-            Err(source) => return ResponseFuture::error(source),
-        };
-
-        if let Some(reason) = self.reason.as_ref() {
-            let header = match request::audit_header(reason) {
-                Ok(header) => header,
-                Err(source) => return ResponseFuture::error(source),
-            };
-
-            request = request.headers(header);
+        match self.try_into_request() {
+            Ok(request) => http.request(request),
+            Err(source) => ResponseFuture::error(source),
         }
-
-        self.http.request(request.build())
     }
 }
 
@@ -284,3 +207,21 @@ impl<'a> AuditLogReason<'a> for CreateGuildChannel<'a> {
         Ok(self)
     }
 }
+
+impl TryIntoRequest for CreateGuildChannel<'_> {
+    fn try_into_request(self) -> Result<Request, HttpError> {
+        let mut request = Request::builder(&Route::CreateChannel {
+            guild_id: self.guild_id.get(),
+        });
+
+        request = request.json(&self.fields)?;
+
+        if let Some(reason) = self.reason.as_ref() {
+            let header = request::audit_header(reason)?;
+
+            request = request.headers(header);
+        }
+
+        Ok(request.build())
+    }
+}
