@@ -1,22 +1,30 @@
 use crate::{
     client::Client,
-    request::{self, AuditLogReason, AuditLogReasonError, Request},
+    error::Error,
+    request::{self, AuditLogReason, AuditLogReasonError, Request, TryIntoRequest},
     response::{marker::EmptyBody, ResponseFuture},
     routing::Route,
 };
-use twilight_model::id::{EmojiId, GuildId};
+use twilight_model::id::{
+    marker::{EmojiMarker, GuildMarker},
+    Id,
+};
 
 /// Delete an emoji in a guild, by id.
 #[must_use = "requests must be configured and executed"]
 pub struct DeleteEmoji<'a> {
-    emoji_id: EmojiId,
-    guild_id: GuildId,
+    emoji_id: Id<EmojiMarker>,
+    guild_id: Id<GuildMarker>,
     http: &'a Client,
     reason: Option<&'a str>,
 }
 
 impl<'a> DeleteEmoji<'a> {
-    pub(crate) const fn new(http: &'a Client, guild_id: GuildId, emoji_id: EmojiId) -> Self {
+    pub(crate) const fn new(
+        http: &'a Client,
+        guild_id: Id<GuildMarker>,
+        emoji_id: Id<EmojiMarker>,
+    ) -> Self {
         Self {
             emoji_id,
             guild_id,
@@ -29,21 +37,12 @@ impl<'a> DeleteEmoji<'a> {
     ///
     /// [`Response`]: crate::response::Response
     pub fn exec(self) -> ResponseFuture<EmptyBody> {
-        let mut request = Request::builder(&Route::DeleteEmoji {
-            emoji_id: self.emoji_id.get(),
-            guild_id: self.guild_id.get(),
-        });
+        let http = self.http;
 
-        if let Some(reason) = self.reason.as_ref() {
-            let header = match request::audit_header(reason) {
-                Ok(header) => header,
-                Err(source) => return ResponseFuture::error(source),
-            };
-
-            request = request.headers(header);
+        match self.try_into_request() {
+            Ok(request) => http.request(request),
+            Err(source) => ResponseFuture::error(source),
         }
-
-        self.http.request(request.build())
     }
 }
 
@@ -54,3 +53,20 @@ impl<'a> AuditLogReason<'a> for DeleteEmoji<'a> {
         Ok(self)
     }
 }
+
+impl TryIntoRequest for DeleteEmoji<'_> {
+    fn try_into_request(self) -> Result<Request, Error> {
+        let mut request = Request::builder(&Route::DeleteEmoji {
+            emoji_id: self.emoji_id.get(),
+            guild_id: self.guild_id.get(),
+        });
+
+        if let Some(reason) = self.reason.as_ref() {
+            let header = request::audit_header(reason)?;
+
+            request = request.headers(header);
+        }
+
+        Ok(request.build())
+    }
+}
