@@ -1,7 +1,16 @@
-use crate::{client::Client, request::Request, response::ResponseFuture, routing::Route};
+use crate::{
+    client::Client,
+    error::Error,
+    request::{Request, TryIntoRequest},
+    response::ResponseFuture,
+    routing::Route,
+};
 use twilight_model::{
     guild::Emoji,
-    id::{EmojiId, GuildId},
+    id::{
+        marker::{EmojiMarker, GuildMarker},
+        Id,
+    },
 };
 
 /// Get an emoji for a guild by the the guild's ID and emoji's ID.
@@ -12,27 +21,31 @@ use twilight_model::{
 ///
 /// ```no_run
 /// use twilight_http::Client;
-/// use twilight_model::id::{EmojiId, GuildId};
+/// use twilight_model::id::Id;
 ///
 /// # #[tokio::main]
 /// # async fn main() -> Result<(), Box<dyn std::error::Error>> {
 /// let client = Client::new("my token".to_owned());
 ///
-/// let guild_id = GuildId::new(50).expect("non zero");
-/// let emoji_id = EmojiId::new(100).expect("non zero");
+/// let guild_id = Id::new(50);
+/// let emoji_id = Id::new(100);
 ///
 /// client.emoji(guild_id, emoji_id).exec().await?;
 /// # Ok(()) }
 /// ```
 #[must_use = "requests must be configured and executed"]
 pub struct GetEmoji<'a> {
-    emoji_id: EmojiId,
-    guild_id: GuildId,
+    emoji_id: Id<EmojiMarker>,
+    guild_id: Id<GuildMarker>,
     http: &'a Client,
 }
 
 impl<'a> GetEmoji<'a> {
-    pub(crate) const fn new(http: &'a Client, guild_id: GuildId, emoji_id: EmojiId) -> Self {
+    pub(crate) const fn new(
+        http: &'a Client,
+        guild_id: Id<GuildMarker>,
+        emoji_id: Id<EmojiMarker>,
+    ) -> Self {
         Self {
             emoji_id,
             guild_id,
@@ -44,11 +57,20 @@ impl<'a> GetEmoji<'a> {
     ///
     /// [`Response`]: crate::response::Response
     pub fn exec(self) -> ResponseFuture<Emoji> {
-        let request = Request::from_route(&Route::GetEmoji {
-            emoji_id: self.emoji_id.get(),
-            guild_id: self.guild_id.get(),
-        });
+        let http = self.http;
 
-        self.http.request(request)
+        match self.try_into_request() {
+            Ok(request) => http.request(request),
+            Err(source) => ResponseFuture::error(source),
+        }
+    }
+}
+
+impl TryIntoRequest for GetEmoji<'_> {
+    fn try_into_request(self) -> Result<Request, Error> {
+        Ok(Request::from_route(&Route::GetEmoji {
+            emoji_id: self.emoji_id.get(),
+            guild_id: self.guild_id.get(),
+        }))
     }
 }
