@@ -1,10 +1,14 @@
 use crate::{
     client::Client,
-    request::Request,
+    error::Error,
+    request::{Request, TryIntoRequest},
     response::{marker::ListBody, ResponseFuture},
     routing::Route,
 };
-use twilight_model::{id::GuildId, invite::Invite};
+use twilight_model::{
+    id::{marker::GuildMarker, Id},
+    invite::Invite,
+};
 
 /// Get information about the invites of a guild.
 ///
@@ -13,12 +17,12 @@ use twilight_model::{id::GuildId, invite::Invite};
 /// [`MANAGE_GUILD`]: twilight_model::guild::Permissions::MANAGE_GUILD
 #[must_use = "requests must be configured and executed"]
 pub struct GetGuildInvites<'a> {
-    guild_id: GuildId,
+    guild_id: Id<GuildMarker>,
     http: &'a Client,
 }
 
 impl<'a> GetGuildInvites<'a> {
-    pub(crate) const fn new(http: &'a Client, guild_id: GuildId) -> Self {
+    pub(crate) const fn new(http: &'a Client, guild_id: Id<GuildMarker>) -> Self {
         Self { guild_id, http }
     }
 
@@ -26,10 +30,19 @@ impl<'a> GetGuildInvites<'a> {
     ///
     /// [`Response`]: crate::response::Response
     pub fn exec(self) -> ResponseFuture<ListBody<Invite>> {
-        let request = Request::from_route(&Route::GetGuildInvites {
-            guild_id: self.guild_id.get(),
-        });
+        let http = self.http;
 
-        self.http.request(request)
+        match self.try_into_request() {
+            Ok(request) => http.request(request),
+            Err(source) => ResponseFuture::error(source),
+        }
+    }
+}
+
+impl TryIntoRequest for GetGuildInvites<'_> {
+    fn try_into_request(self) -> Result<Request, Error> {
+        Ok(Request::from_route(&Route::GetGuildInvites {
+            guild_id: self.guild_id.get(),
+        }))
     }
 }
