@@ -1,25 +1,29 @@
 use crate::{
     client::Client,
-    request::{self, AuditLogReason, AuditLogReasonError, Request},
+    error::Error,
+    request::{self, AuditLogReason, AuditLogReasonError, Request, TryIntoRequest},
     response::{marker::EmptyBody, ResponseFuture},
     routing::Route,
 };
-use twilight_model::id::{GuildId, IntegrationId};
+use twilight_model::id::{
+    marker::{GuildMarker, IntegrationMarker},
+    Id,
+};
 
 /// Delete an integration for a guild, by the integration's id.
 #[must_use = "requests must be configured and executed"]
 pub struct DeleteGuildIntegration<'a> {
-    guild_id: GuildId,
+    guild_id: Id<GuildMarker>,
     http: &'a Client,
-    integration_id: IntegrationId,
+    integration_id: Id<IntegrationMarker>,
     reason: Option<&'a str>,
 }
 
 impl<'a> DeleteGuildIntegration<'a> {
     pub(crate) const fn new(
         http: &'a Client,
-        guild_id: GuildId,
-        integration_id: IntegrationId,
+        guild_id: Id<GuildMarker>,
+        integration_id: Id<IntegrationMarker>,
     ) -> Self {
         Self {
             guild_id,
@@ -33,21 +37,12 @@ impl<'a> DeleteGuildIntegration<'a> {
     ///
     /// [`Response`]: crate::response::Response
     pub fn exec(self) -> ResponseFuture<EmptyBody> {
-        let mut request = Request::builder(&Route::DeleteGuildIntegration {
-            guild_id: self.guild_id.get(),
-            integration_id: self.integration_id.get(),
-        });
+        let http = self.http;
 
-        if let Some(reason) = self.reason.as_ref() {
-            let header = match request::audit_header(reason) {
-                Ok(header) => header,
-                Err(source) => return ResponseFuture::error(source),
-            };
-
-            request = request.headers(header);
+        match self.try_into_request() {
+            Ok(request) => http.request(request),
+            Err(source) => ResponseFuture::error(source),
         }
-
-        self.http.request(request.build())
     }
 }
 
@@ -58,3 +53,20 @@ impl<'a> AuditLogReason<'a> for DeleteGuildIntegration<'a> {
         Ok(self)
     }
 }
+
+impl TryIntoRequest for DeleteGuildIntegration<'_> {
+    fn try_into_request(self) -> Result<Request, Error> {
+        let mut request = Request::builder(&Route::DeleteGuildIntegration {
+            guild_id: self.guild_id.get(),
+            integration_id: self.integration_id.get(),
+        });
+
+        if let Some(reason) = self.reason.as_ref() {
+            let header = request::audit_header(reason)?;
+
+            request = request.headers(header);
+        }
+
+        Ok(request.build())
+    }
+}
