@@ -1,10 +1,14 @@
 use crate::{
     client::Client,
-    request::{self, AuditLogReason, AuditLogReasonError, Request},
+    error::Error,
+    request::{self, AuditLogReason, AuditLogReasonError, Request, TryIntoRequest},
     response::{marker::EmptyBody, ResponseFuture},
     routing::Route,
 };
-use twilight_model::id::{GuildId, RoleId, UserId};
+use twilight_model::id::{
+    marker::{GuildMarker, RoleMarker, UserMarker},
+    Id,
+};
 
 /// Add a role to a member in a guild.
 ///
@@ -14,15 +18,15 @@ use twilight_model::id::{GuildId, RoleId, UserId};
 ///
 /// ```no_run
 /// use twilight_http::{request::AuditLogReason, Client};
-/// use twilight_model::id::{GuildId, RoleId, UserId};
+/// use twilight_model::id::Id;
 ///
 /// # #[tokio::main]
 /// # async fn main() -> Result<(), Box<dyn std::error::Error>> {
 /// let client = Client::new("my token".to_owned());
 ///
-/// let guild_id = GuildId::new(1).expect("non zero");
-/// let role_id = RoleId::new(2).expect("non zero");
-/// let user_id = UserId::new(3).expect("non zero");
+/// let guild_id = Id::new(1);
+/// let role_id = Id::new(2);
+/// let user_id = Id::new(3);
 ///
 /// client.add_guild_member_role(guild_id, user_id, role_id)
 ///     .reason("test")?
@@ -32,19 +36,19 @@ use twilight_model::id::{GuildId, RoleId, UserId};
 /// ```
 #[must_use = "requests must be configured and executed"]
 pub struct AddRoleToMember<'a> {
-    guild_id: GuildId,
+    guild_id: Id<GuildMarker>,
     http: &'a Client,
-    role_id: RoleId,
-    user_id: UserId,
+    role_id: Id<RoleMarker>,
+    user_id: Id<UserMarker>,
     reason: Option<&'a str>,
 }
 
 impl<'a> AddRoleToMember<'a> {
     pub(crate) const fn new(
         http: &'a Client,
-        guild_id: GuildId,
-        user_id: UserId,
-        role_id: RoleId,
+        guild_id: Id<GuildMarker>,
+        user_id: Id<UserMarker>,
+        role_id: Id<RoleMarker>,
     ) -> Self {
         Self {
             guild_id,
@@ -59,22 +63,12 @@ impl<'a> AddRoleToMember<'a> {
     ///
     /// [`Response`]: crate::response::Response
     pub fn exec(self) -> ResponseFuture<EmptyBody> {
-        let mut request = Request::builder(&Route::AddMemberRole {
-            guild_id: self.guild_id.get(),
-            role_id: self.role_id.get(),
-            user_id: self.user_id.get(),
-        });
+        let http = self.http;
 
-        if let Some(reason) = self.reason.as_ref() {
-            let header = match request::audit_header(reason) {
-                Ok(header) => header,
-                Err(source) => return ResponseFuture::error(source),
-            };
-
-            request = request.headers(header);
+        match self.try_into_request() {
+            Ok(request) => http.request(request),
+            Err(source) => ResponseFuture::error(source),
         }
-
-        self.http.request(request.build())
     }
 }
 
@@ -85,3 +79,21 @@ impl<'a> AuditLogReason<'a> for AddRoleToMember<'a> {
         Ok(self)
     }
 }
+
+impl TryIntoRequest for AddRoleToMember<'_> {
+    fn try_into_request(self) -> Result<Request, Error> {
+        let mut request = Request::builder(&Route::AddMemberRole {
+            guild_id: self.guild_id.get(),
+            role_id: self.role_id.get(),
+            user_id: self.user_id.get(),
+        });
+
+        if let Some(reason) = self.reason.as_ref() {
+            let header = request::audit_header(reason)?;
+
+            request = request.headers(header);
+        }
+
+        Ok(request.build())
+    }
+}
