@@ -1,22 +1,30 @@
 use crate::{
     client::Client,
-    request::{self, AuditLogReason, AuditLogReasonError, Request},
+    error::Error,
+    request::{self, AuditLogReason, AuditLogReasonError, Request, TryIntoRequest},
     response::{marker::EmptyBody, ResponseFuture},
     routing::Route,
 };
-use twilight_model::id::{GuildId, RoleId};
+use twilight_model::id::{
+    marker::{GuildMarker, RoleMarker},
+    Id,
+};
 
 /// Delete a role in a guild, by id.
 #[must_use = "requests must be configured and executed"]
 pub struct DeleteRole<'a> {
-    guild_id: GuildId,
+    guild_id: Id<GuildMarker>,
     http: &'a Client,
-    role_id: RoleId,
+    role_id: Id<RoleMarker>,
     reason: Option<&'a str>,
 }
 
 impl<'a> DeleteRole<'a> {
-    pub(crate) const fn new(http: &'a Client, guild_id: GuildId, role_id: RoleId) -> Self {
+    pub(crate) const fn new(
+        http: &'a Client,
+        guild_id: Id<GuildMarker>,
+        role_id: Id<RoleMarker>,
+    ) -> Self {
         Self {
             guild_id,
             http,
@@ -29,21 +37,12 @@ impl<'a> DeleteRole<'a> {
     ///
     /// [`Response`]: crate::response::Response
     pub fn exec(self) -> ResponseFuture<EmptyBody> {
-        let mut request = Request::builder(&Route::DeleteRole {
-            guild_id: self.guild_id.get(),
-            role_id: self.role_id.get(),
-        });
+        let http = self.http;
 
-        if let Some(reason) = &self.reason {
-            let header = match request::audit_header(reason) {
-                Ok(header) => header,
-                Err(source) => return ResponseFuture::error(source),
-            };
-
-            request = request.headers(header);
+        match self.try_into_request() {
+            Ok(request) => http.request(request),
+            Err(source) => ResponseFuture::error(source),
         }
-
-        self.http.request(request.build())
     }
 }
 
@@ -54,3 +53,20 @@ impl<'a> AuditLogReason<'a> for DeleteRole<'a> {
         Ok(self)
     }
 }
+
+impl TryIntoRequest for DeleteRole<'_> {
+    fn try_into_request(self) -> Result<Request, Error> {
+        let mut request = Request::builder(&Route::DeleteRole {
+            guild_id: self.guild_id.get(),
+            role_id: self.role_id.get(),
+        });
+
+        if let Some(reason) = &self.reason {
+            let header = request::audit_header(reason)?;
+
+            request = request.headers(header);
+        }
+
+        Ok(request.build())
+    }
+}
