@@ -1,13 +1,17 @@
 use crate::{
     client::Client,
-    request::{self, AuditLogReason, AuditLogReasonError, NullableField, Request},
+    error::Error,
+    request::{self, AuditLogReason, AuditLogReasonError, NullableField, Request, TryIntoRequest},
     response::ResponseFuture,
     routing::Route,
 };
 use serde::Serialize;
 use twilight_model::{
     guild::{Permissions, Role},
-    id::{GuildId, RoleId},
+    id::{
+        marker::{GuildMarker, RoleMarker},
+        Id,
+    },
 };
 
 #[derive(Serialize)]
@@ -32,14 +36,18 @@ struct UpdateRoleFields<'a> {
 #[must_use = "requests must be configured and executed"]
 pub struct UpdateRole<'a> {
     fields: UpdateRoleFields<'a>,
-    guild_id: GuildId,
+    guild_id: Id<GuildMarker>,
     http: &'a Client,
-    role_id: RoleId,
+    role_id: Id<RoleMarker>,
     reason: Option<&'a str>,
 }
 
 impl<'a> UpdateRole<'a> {
-    pub(crate) const fn new(http: &'a Client, guild_id: GuildId, role_id: RoleId) -> Self {
+    pub(crate) const fn new(
+        http: &'a Client,
+        guild_id: Id<GuildMarker>,
+        role_id: Id<RoleMarker>,
+    ) -> Self {
         Self {
             fields: UpdateRoleFields {
                 color: None,
@@ -116,26 +124,12 @@ impl<'a> UpdateRole<'a> {
     ///
     /// [`Response`]: crate::response::Response
     pub fn exec(self) -> ResponseFuture<Role> {
-        let mut request = Request::builder(&Route::UpdateRole {
-            guild_id: self.guild_id.get(),
-            role_id: self.role_id.get(),
-        });
+        let http = self.http;
 
-        request = match request.json(&self.fields) {
-            Ok(request) => request,
-            Err(source) => return ResponseFuture::error(source),
-        };
-
-        if let Some(reason) = &self.reason {
-            let header = match request::audit_header(reason) {
-                Ok(header) => header,
-                Err(source) => return ResponseFuture::error(source),
-            };
-
-            request = request.headers(header);
+        match self.try_into_request() {
+            Ok(request) => http.request(request),
+            Err(source) => ResponseFuture::error(source),
         }
-
-        self.http.request(request.build())
     }
 }
 
@@ -146,3 +140,22 @@ impl<'a> AuditLogReason<'a> for UpdateRole<'a> {
         Ok(self)
     }
 }
+
+impl TryIntoRequest for UpdateRole<'_> {
+    fn try_into_request(self) -> Result<Request, Error> {
+        let mut request = Request::builder(&Route::UpdateRole {
+            guild_id: self.guild_id.get(),
+            role_id: self.role_id.get(),
+        });
+
+        request = request.json(&self.fields)?;
+
+        if let Some(reason) = &self.reason {
+            let header = request::audit_header(reason)?;
+
+            request = request.headers(header);
+        }
+
+        Ok(request.build())
+    }
+}
