@@ -1,10 +1,14 @@
 use crate::{
     client::Client,
-    request::Request,
+    error::Error,
+    request::{Request, TryIntoRequest},
     response::{marker::EmptyBody, ResponseFuture},
     routing::Route,
 };
-use twilight_model::{channel::message::sticker::StickerId, id::GuildId};
+use twilight_model::id::{
+    marker::{GuildMarker, StickerMarker},
+    Id,
+};
 
 /// Deletes a guild sticker by the ID of the guild and its ID.
 ///
@@ -12,17 +16,14 @@ use twilight_model::{channel::message::sticker::StickerId, id::GuildId};
 ///
 /// ```no_run
 /// use twilight_http::Client;
-/// use twilight_model::{
-///     channel::message::sticker::StickerId,
-///     id::GuildId,
-/// };
+/// use twilight_model::id::Id;
 ///
 /// # #[tokio::main]
 /// # async fn main() -> Result<(), Box<dyn std::error::Error>> {
 /// let client = Client::new("my token".to_owned());
 ///
-/// let guild_id = GuildId::new(1).expect("non zero");
-/// let sticker_id = StickerId::new(2).expect("non zero");
+/// let guild_id = Id::new(1);
+/// let sticker_id = Id::new(2);
 ///
 /// client
 ///     .delete_guild_sticker(guild_id, sticker_id)
@@ -31,13 +32,17 @@ use twilight_model::{channel::message::sticker::StickerId, id::GuildId};
 /// # Ok(()) }
 /// ```
 pub struct DeleteGuildSticker<'a> {
-    guild_id: GuildId,
+    guild_id: Id<GuildMarker>,
     http: &'a Client,
-    sticker_id: StickerId,
+    sticker_id: Id<StickerMarker>,
 }
 
 impl<'a> DeleteGuildSticker<'a> {
-    pub(crate) const fn new(http: &'a Client, guild_id: GuildId, sticker_id: StickerId) -> Self {
+    pub(crate) const fn new(
+        http: &'a Client,
+        guild_id: Id<GuildMarker>,
+        sticker_id: Id<StickerMarker>,
+    ) -> Self {
         Self {
             guild_id,
             http,
@@ -49,11 +54,20 @@ impl<'a> DeleteGuildSticker<'a> {
     ///
     /// [`Response`]: crate::response::Response
     pub fn exec(self) -> ResponseFuture<EmptyBody> {
-        let request = Request::from_route(&Route::DeleteGuildSticker {
-            guild_id: self.guild_id.get(),
-            sticker_id: self.sticker_id.get(),
-        });
+        let http = self.http;
 
-        self.http.request(request)
+        match self.try_into_request() {
+            Ok(request) => http.request(request),
+            Err(source) => ResponseFuture::error(source),
+        }
+    }
+}
+
+impl TryIntoRequest for DeleteGuildSticker<'_> {
+    fn try_into_request(self) -> Result<Request, Error> {
+        Ok(Request::from_route(&Route::DeleteGuildSticker {
+            guild_id: self.guild_id.get(),
+            sticker_id: self.sticker_id.get(),
+        }))
     }
 }
