@@ -1,15 +1,21 @@
-use super::{StickerValidationError, StickerValidationErrorType};
 use crate::{
     client::Client,
     error::Error as HttpError,
-    request::{validate_inner, AuditLogReason, AuditLogReasonError, Request},
+    request::{AuditLogReason, AuditLogReasonError, Request, TryIntoRequest},
     response::ResponseFuture,
     routing::Route,
 };
 use serde::Serialize;
 use twilight_model::{
-    channel::message::sticker::{Sticker, StickerId},
-    id::GuildId,
+    channel::message::sticker::Sticker,
+    id::{
+        marker::{GuildMarker, StickerMarker},
+        Id,
+    },
+};
+use twilight_validate::sticker::{
+    description as validate_description, name as validate_name, tags as validate_tags,
+    StickerValidationError,
 };
 
 #[derive(Serialize)]
@@ -25,17 +31,14 @@ struct UpdateGuildStickerFields<'a> {
 ///
 /// ```no_run
 /// use twilight_http::Client;
-/// use twilight_model::{
-///     channel::message::sticker::StickerId,
-///     id::GuildId,
-/// };
+/// use twilight_model::id::Id;
 ///
 /// # #[tokio::main]
 /// # async fn main() -> Result<(), Box<dyn std::error::Error>> {
 /// let client = Client::new("my token".to_owned());
 ///
-/// let guild_id = GuildId::new(1).expect("non zero");
-/// let sticker_id = StickerId::new(2).expect("non zero");
+/// let guild_id = Id::new(1);
+/// let sticker_id = Id::new(2);
 /// let sticker = client
 ///     .update_guild_sticker(guild_id, sticker_id)
 ///     .description("new description")?
@@ -49,14 +52,18 @@ struct UpdateGuildStickerFields<'a> {
 /// ```
 pub struct UpdateGuildSticker<'a> {
     fields: UpdateGuildStickerFields<'a>,
-    guild_id: GuildId,
+    guild_id: Id<GuildMarker>,
     http: &'a Client,
     reason: Option<&'a str>,
-    sticker_id: StickerId,
+    sticker_id: Id<StickerMarker>,
 }
 
 impl<'a> UpdateGuildSticker<'a> {
-    pub(crate) const fn new(http: &'a Client, guild_id: GuildId, sticker_id: StickerId) -> Self {
+    pub(crate) const fn new(
+        http: &'a Client,
+        guild_id: Id<GuildMarker>,
+        sticker_id: Id<StickerMarker>,
+    ) -> Self {
         Self {
             guild_id,
             fields: UpdateGuildStickerFields {
@@ -71,11 +78,7 @@ impl<'a> UpdateGuildSticker<'a> {
     }
 
     pub fn description(mut self, description: &'a str) -> Result<Self, StickerValidationError> {
-        if !validate_inner::sticker_description(description) {
-            return Err(StickerValidationError {
-                kind: StickerValidationErrorType::DescriptionInvalid,
-            });
-        }
+        validate_description(description)?;
 
         self.fields.description = Some(description);
 
@@ -83,11 +86,7 @@ impl<'a> UpdateGuildSticker<'a> {
     }
 
     pub fn name(mut self, name: &'a str) -> Result<Self, StickerValidationError> {
-        if !validate_inner::sticker_name(name) {
-            return Err(StickerValidationError {
-                kind: StickerValidationErrorType::NameInvalid,
-            });
-        }
+        validate_name(name)?;
 
         self.fields.name = Some(name);
 
@@ -95,33 +94,21 @@ impl<'a> UpdateGuildSticker<'a> {
     }
 
     pub fn tags(mut self, tags: &'a str) -> Result<Self, StickerValidationError> {
-        if !validate_inner::sticker_tags(tags) {
-            return Err(StickerValidationError {
-                kind: StickerValidationErrorType::TagsInvalid,
-            });
-        }
+        validate_tags(tags)?;
 
         self.fields.tags = Some(tags);
 
         Ok(self)
     }
 
-    fn request(&self) -> Result<Request, HttpError> {
-        let request = Request::builder(&Route::UpdateGuildSticker {
-            guild_id: self.guild_id.get(),
-            sticker_id: self.sticker_id.get(),
-        })
-        .json(&self.fields)?;
-
-        Ok(request.build())
-    }
-
     /// Execute the request, returning a future resolving to a [`Response`].
     ///
     /// [`Response`]: crate::response::Response
     pub fn exec(self) -> ResponseFuture<Sticker> {
-        match self.request() {
-            Ok(request) => self.http.request(request),
+        let http = self.http;
+
+        match self.try_into_request() {
+            Ok(request) => http.request(request),
             Err(source) => ResponseFuture::error(source),
         }
     }
@@ -134,3 +121,15 @@ impl<'a> AuditLogReason<'a> for UpdateGuildSticker<'a> {
         Ok(self)
     }
 }
+
+impl TryIntoRequest for UpdateGuildSticker<'_> {
+    fn try_into_request(self) -> Result<Request, HttpError> {
+        let request = Request::builder(&Route::UpdateGuildSticker {
+            guild_id: self.guild_id.get(),
+            sticker_id: self.sticker_id.get(),
+        })
+        .json(&self.fields)?;
+
+        Ok(request.build())
+    }
+}
