@@ -1,72 +1,27 @@
 use crate::{
     client::Client,
-    request::{self, validate_inner, AuditLogReason, AuditLogReasonError, NullableField, Request},
+    error::Error as HttpError,
+    request::{self, AuditLogReason, AuditLogReasonError, NullableField, Request, TryIntoRequest},
     response::ResponseFuture,
     routing::Route,
 };
 use serde::Serialize;
-use std::{
-    error::Error,
-    fmt::{Display, Formatter, Result as FmtResult},
-};
 use twilight_model::{
     guild::{
         DefaultMessageNotificationLevel, ExplicitContentFilter, PartialGuild, SystemChannelFlags,
         VerificationLevel,
     },
-    id::{ChannelId, GuildId, UserId},
+    id::{
+        marker::{ChannelMarker, GuildMarker, UserMarker},
+        Id,
+    },
 };
-
-/// The error returned when the guild can not be updated as configured.
-#[derive(Debug)]
-pub struct UpdateGuildError {
-    kind: UpdateGuildErrorType,
-}
-
-impl UpdateGuildError {
-    /// Immutable reference to the type of error that occurred.
-    #[must_use = "retrieving the type has no effect if left unused"]
-    pub const fn kind(&self) -> &UpdateGuildErrorType {
-        &self.kind
-    }
-
-    /// Consume the error, returning the source error if there is any.
-    #[allow(clippy::unused_self)]
-    #[must_use = "consuming the error and retrieving the source has no effect if left unused"]
-    pub fn into_source(self) -> Option<Box<dyn Error + Send + Sync>> {
-        None
-    }
-
-    /// Consume the error, returning the owned error type and the source error.
-    #[must_use = "consuming the error into its parts has no effect if left unused"]
-    pub fn into_parts(self) -> (UpdateGuildErrorType, Option<Box<dyn Error + Send + Sync>>) {
-        (self.kind, None)
-    }
-}
-
-impl Display for UpdateGuildError {
-    fn fmt(&self, f: &mut Formatter<'_>) -> FmtResult {
-        match &self.kind {
-            UpdateGuildErrorType::NameInvalid => f.write_str("the name's length is invalid"),
-        }
-    }
-}
-
-impl Error for UpdateGuildError {}
-
-/// Type of [`UpdateGuildError`] that occurred.
-#[derive(Debug)]
-#[non_exhaustive]
-pub enum UpdateGuildErrorType {
-    /// The name length is either fewer than 2 UTF-16 characters or more than 100 UTF-16
-    /// characters.
-    NameInvalid,
-}
+use twilight_validate::request::{guild_name as validate_guild_name, ValidationError};
 
 #[derive(Serialize)]
 struct UpdateGuildFields<'a> {
     #[serde(skip_serializing_if = "Option::is_none")]
-    afk_channel_id: Option<NullableField<ChannelId>>,
+    afk_channel_id: Option<NullableField<Id<ChannelMarker>>>,
     #[serde(skip_serializing_if = "Option::is_none")]
     afk_timeout: Option<u64>,
     #[serde(skip_serializing_if = "Option::is_none")]
@@ -84,19 +39,19 @@ struct UpdateGuildFields<'a> {
     #[serde(skip_serializing_if = "Option::is_none")]
     name: Option<&'a str>,
     #[serde(skip_serializing_if = "Option::is_none")]
-    owner_id: Option<UserId>,
+    owner_id: Option<Id<UserMarker>>,
     #[serde(skip_serializing_if = "Option::is_none")]
     splash: Option<NullableField<&'a str>>,
     #[serde(skip_serializing_if = "Option::is_none")]
-    system_channel_id: Option<NullableField<ChannelId>>,
+    system_channel_id: Option<NullableField<Id<ChannelMarker>>>,
     #[serde(skip_serializing_if = "Option::is_none")]
     system_channel_flags: Option<NullableField<SystemChannelFlags>>,
     #[serde(skip_serializing_if = "Option::is_none")]
     verification_level: Option<NullableField<VerificationLevel>>,
     #[serde(skip_serializing_if = "Option::is_none")]
-    rules_channel_id: Option<NullableField<ChannelId>>,
+    rules_channel_id: Option<NullableField<Id<ChannelMarker>>>,
     #[serde(skip_serializing_if = "Option::is_none")]
-    public_updates_channel_id: Option<NullableField<ChannelId>>,
+    public_updates_channel_id: Option<NullableField<Id<ChannelMarker>>>,
     #[serde(skip_serializing_if = "Option::is_none")]
     preferred_locale: Option<NullableField<&'a str>>,
     #[serde(skip_serializing_if = "Option::is_none")]
@@ -105,19 +60,19 @@ struct UpdateGuildFields<'a> {
 
 /// Update a guild.
 ///
-/// All endpoints are optional. Refer to [the discord docs] for more information.
+/// All endpoints are optional. See [Discord Docs/Modify Guild].
 ///
-/// [the discord docs]: https://discord.com/developers/docs/resources/guild#modify-guild
+/// [Discord Docs/Modify Guild]: https://discord.com/developers/docs/resources/guild#modify-guild
 #[must_use = "requests must be configured and executed"]
 pub struct UpdateGuild<'a> {
     fields: UpdateGuildFields<'a>,
-    guild_id: GuildId,
+    guild_id: Id<GuildMarker>,
     http: &'a Client,
     reason: Option<&'a str>,
 }
 
 impl<'a> UpdateGuild<'a> {
-    pub(crate) const fn new(http: &'a Client, guild_id: GuildId) -> Self {
+    pub(crate) const fn new(http: &'a Client, guild_id: Id<GuildMarker>) -> Self {
         Self {
             fields: UpdateGuildFields {
                 afk_channel_id: None,
@@ -146,7 +101,7 @@ impl<'a> UpdateGuild<'a> {
     }
 
     /// Set the voice channel where AFK voice users are sent.
-    pub const fn afk_channel_id(mut self, afk_channel_id: Option<ChannelId>) -> Self {
+    pub const fn afk_channel_id(mut self, afk_channel_id: Option<Id<ChannelMarker>>) -> Self {
         self.fields.afk_channel_id = Some(NullableField(afk_channel_id));
 
         self
@@ -171,10 +126,10 @@ impl<'a> UpdateGuild<'a> {
         self
     }
 
-    /// Set the default message notification level. Refer to [the discord docs] for more
-    /// information.
+    /// Set the default message notification level. See
+    /// [Discord Docs/Create Guild] for more information.
     ///
-    /// [the discord docs]: https://discord.com/developers/docs/resources/guild#create-guild
+    /// [Discord Docs/Create Guild]: https://discord.com/developers/docs/resources/guild#create-guild
     pub const fn default_message_notifications(
         mut self,
         default_message_notifications: Option<DefaultMessageNotificationLevel>,
@@ -213,11 +168,11 @@ impl<'a> UpdateGuild<'a> {
 
     /// Set the icon.
     ///
-    /// This must be a Data URI, in the form of `data:image/{type};base64,{data}` where `{type}` is
-    /// the image MIME type and `{data}` is the base64-encoded image. Refer to [the discord docs]
-    /// for more information.
+    /// This must be a Data URI, in the form of
+    /// `data:image/{type};base64,{data}` where `{type}` is the image MIME type
+    /// and `{data}` is the base64-encoded image. See [Discord Docs/Image Data].
     ///
-    /// [the discord docs]: https://discord.com/developers/docs/reference#image-data
+    /// [Discord Docs/Image Data]: https://discord.com/developers/docs/reference#image-data
     pub const fn icon(mut self, icon: Option<&'a str>) -> Self {
         self.fields.icon = Some(NullableField(icon));
 
@@ -231,14 +186,12 @@ impl<'a> UpdateGuild<'a> {
     ///
     /// # Errors
     ///
-    /// Returns an [`UpdateGuildErrorType::NameInvalid`] error type if the name
-    /// length is too short or too long.
-    pub fn name(mut self, name: &'a str) -> Result<Self, UpdateGuildError> {
-        if !validate_inner::guild_name(name) {
-            return Err(UpdateGuildError {
-                kind: UpdateGuildErrorType::NameInvalid,
-            });
-        }
+    /// Returns an error of type [`GuildName`] if the name length is too short
+    /// or too long.
+    ///
+    /// [`GuildName`]: twilight_validate::request::ValidationErrorType::GuildName
+    pub fn name(mut self, name: &'a str) -> Result<Self, ValidationError> {
+        validate_guild_name(name)?;
 
         self.fields.name.replace(name);
 
@@ -248,7 +201,7 @@ impl<'a> UpdateGuild<'a> {
     /// Transfer ownership to another user.
     ///
     /// Only works if the current user is the owner.
-    pub const fn owner_id(mut self, owner_id: UserId) -> Self {
+    pub const fn owner_id(mut self, owner_id: Id<UserMarker>) -> Self {
         self.fields.owner_id = Some(owner_id);
 
         self
@@ -264,7 +217,7 @@ impl<'a> UpdateGuild<'a> {
     }
 
     /// Set the channel where events such as welcome messages are posted.
-    pub const fn system_channel(mut self, system_channel_id: Option<ChannelId>) -> Self {
+    pub const fn system_channel(mut self, system_channel_id: Option<Id<ChannelMarker>>) -> Self {
         self.fields.system_channel_id = Some(NullableField(system_channel_id));
 
         self
@@ -282,10 +235,10 @@ impl<'a> UpdateGuild<'a> {
 
     /// Set the rules channel.
     ///
-    /// Requires the guild to be `PUBLIC`. Refer to [the discord docs] for more information.
+    /// Requires the guild to be `PUBLIC`. See [Discord Docs/Modify Guild].
     ///
-    /// [the discord docs]: https://discord.com/developers/docs/resources/guild#modify-guild
-    pub const fn rules_channel(mut self, rules_channel_id: Option<ChannelId>) -> Self {
+    /// [Discord Docs/Modify Guild]: https://discord.com/developers/docs/resources/guild#modify-guild
+    pub const fn rules_channel(mut self, rules_channel_id: Option<Id<ChannelMarker>>) -> Self {
         self.fields.rules_channel_id = Some(NullableField(rules_channel_id));
 
         self
@@ -296,7 +249,7 @@ impl<'a> UpdateGuild<'a> {
     /// Requires the guild to be `PUBLIC`.
     pub const fn public_updates_channel(
         mut self,
-        public_updates_channel_id: Option<ChannelId>,
+        public_updates_channel_id: Option<Id<ChannelMarker>>,
     ) -> Self {
         self.fields.public_updates_channel_id = Some(NullableField(public_updates_channel_id));
 
@@ -312,9 +265,11 @@ impl<'a> UpdateGuild<'a> {
         self
     }
 
-    /// Set the verification level. Refer to [the discord docs] for more information.
+    /// Set the verification level.
     ///
-    /// [the discord docs]: https://discord.com/developers/docs/resources/guild#guild-object-verification-level
+    /// See [Discord Docs/Guild Object].
+    ///
+    /// [Discord Docs/Guild Object]: https://discord.com/developers/docs/resources/guild#guild-object-verification-level
     pub const fn verification_level(
         mut self,
         verification_level: Option<VerificationLevel>,
@@ -338,25 +293,12 @@ impl<'a> UpdateGuild<'a> {
     ///
     /// [`Response`]: crate::response::Response
     pub fn exec(self) -> ResponseFuture<PartialGuild> {
-        let mut request = Request::builder(&Route::UpdateGuild {
-            guild_id: self.guild_id.get(),
-        });
+        let http = self.http;
 
-        request = match request.json(&self.fields) {
-            Ok(request) => request,
-            Err(source) => return ResponseFuture::error(source),
-        };
-
-        if let Some(reason) = &self.reason {
-            let header = match request::audit_header(reason) {
-                Ok(header) => header,
-                Err(source) => return ResponseFuture::error(source),
-            };
-
-            request = request.headers(header);
+        match self.try_into_request() {
+            Ok(request) => http.request(request),
+            Err(source) => ResponseFuture::error(source),
         }
-
-        self.http.request(request.build())
     }
 }
 
@@ -367,3 +309,21 @@ impl<'a> AuditLogReason<'a> for UpdateGuild<'a> {
         Ok(self)
     }
 }
+
+impl TryIntoRequest for UpdateGuild<'_> {
+    fn try_into_request(self) -> Result<Request, HttpError> {
+        let mut request = Request::builder(&Route::UpdateGuild {
+            guild_id: self.guild_id.get(),
+        });
+
+        request = request.json(&self.fields)?;
+
+        if let Some(reason) = &self.reason {
+            let header = request::audit_header(reason)?;
+
+            request = request.headers(header);
+        }
+
+        Ok(request.build())
+    }
+}
